/* $Id: ComponentBuilderImpl.java,v 1.5 2002/10/08 16:14:11 hnetynka Exp $ */
package SOFA.Component.DCUP;

/** Base implementation of the component builder.
  *
  * @author Petr Hnetynka
  */
abstract public class ComponentBuilderImpl implements ComponentBuilder {
  /** Component manager. */
  protected SOFA.Component.ComponentManager cm;
  /** Deployment desciptor of the component. */
  protected SOFA.SOFAnode.Run.Deployment.DeploymentDescriptor dd;
  /** If component can be updated. */
  protected boolean updatable;
  /** Full instance name */
  protected String fullInstanceName;

  /** Flag if component is stopped. */
  public boolean stopped;

  public void initialize(SOFA.Component.ComponentManager cm, SOFA.SOFAnode.Run.Deployment.DeploymentDescriptor dd) throws SOFA.Component.ComponentLifecycleException {
    this.cm = cm;
    this.dd = dd;
    fullInstanceName = dd.getInstanceName()+":"+dd.getInstanceId();
    updatable = false;
    stopped = false;
  }
  
  /** Creates connector to component manager from reference to it.
    * @param ref   reference to CM
    * @return      connector to CM
    * @exception SOFA.Component.InstantiationException   prob. bad reference
    */
  protected DCUPComponentManager getCMConnectorFromRef(SOFA.Connector.Reference ref) throws SOFA.Component.InstantiationException {
    SOFA.Component.DCUP.DCUPComponentManager aplcm = null;
    try {
      aplcm = (SOFA.Component.DCUP.DCUPComponentManager) SOFA.Connector.Boot.DCUPComponentManagerConnector.createClt(ref);
    } catch (SOFA.Connector.ConnectorException e) {
      throw new SOFA.Component.InstantiationException("Can't create connector to component manger", e);
    }
    return aplcm;
  }

  public boolean isUpdatable() {
    return updatable;
  }

  public boolean isStopped() {
    return stopped;
  }
}
