/*
 * IDLWriter.java
 *
 * Created on 7. duben 2002, 10:23
 */

package SOFA.Connector.EEG.CodeWriter;

import SOFA.SOFAnode.Made.TIR.*;
import SOFA.SOFAnode.Made.TIR.Access.TIRAccessMethods;

/**
 *
 * @author  ghort
 * @version 
 */
public class IDLWriter {

	protected InterfaceDef inIface;
	
	protected String outFile;
	protected String outInterface;
	protected String outPackage;

	/** Creates new IDLWriter */
	public IDLWriter(String iface) throws CodeWriterException {
	 	try {
			inIface=(InterfaceDef)TIRAccessMethods.lookupCDLContained(SOFA.Connector.TIRAccess.TIRAccess.repository,iface);
		} catch (Exception e) {
			throw new CodeWriterException("Can't access interface '"+iface+"'.",e);
		}
	}

	public void setOut(String outFile, String outPackage, String outInterface) {
		this.outFile=outFile;
		this.outInterface=outInterface;
		this.outPackage=outPackage;
	}

	public void write(IDLWriterDirector director) throws CodeWriterException {
		java.io.PrintWriter out;
		try {
			out=new java.io.PrintWriter(new java.io.BufferedWriter(new java.io.FileWriter(outFile)));
		} catch (Exception e) {
			throw new CodeWriterException("Can't open output file.",e);
		}

		try {
			int i;
			Contained[] mts=TIRAccessMethods.getOperAttrOfInterface(inIface);

			java.util.StringTokenizer mods=new java.util.StringTokenizer(outPackage,".");
			int modsCount=mods.countTokens();
			while (mods.hasMoreTokens()) {
				out.println("module "+mods.nextToken()+" {");
			}
			
			out.println();
			out.print("interface "+outInterface);
			
			String[] exts=director.getExtends();
			if (exts!=null) {
				out.print(": ");
				for (i=0;i<exts.length;i++) {
					if (i>0) {
						out.print(", ");
					}
					out.print(exts[i]);
				}
			}
			
			out.println(" {");
			out.println();

			String ifaceBody=director.getInterfaceBody();
			if (ifaceBody!=null) {
				out.println(ifaceBody);
			}
			
			for (i=0;i<mts.length;i++) {
				String operationDef=director.getOperation((OperationDef)mts[i]);
				
				if (operationDef!=null) {
					out.println("\t"+operationDef+";");
				}
			}
			
			out.println("};");
			out.println();

			while (modsCount-->0) {
				out.println("};");
			}
			
		} catch (IDLWriterDirectorException e) {
			throw new CodeWriterException("Troubles with IDL writer director.",e);
		} catch (java.rmi.RemoteException e) {
			throw new CodeWriterException("Can't access TIR.",e);
		} catch (SOFA.SOFAnode.Made.TIR.TIRExceptLock e) {
			throw new CodeWriterException("TIR is locked.",e);
		}

		out.close();
	}
}
