/*
 * ElementDescription.java
 *
 * Created on 2. duben 2002, 23:08
 */

package SOFA.Connector.EEG.EEM;

import org.w3c.dom.*;

/**
 *
 * @author  ghort
 * @version 
 */
public class ElementDescription {

	public String name;
	public String type;
	public String mangled;

	public SOFA.Connector.Property conditions[];

	public java.util.LinkedList make; // of ElementDescriptionCommand or ElementDescriptionBuild

	/** Creates new ElementDescription */
	public ElementDescription() {
	}

	static public ElementDescription fromXML(org.w3c.dom.Element rootElement) throws ElementDescriptionException {
		if (!rootElement.getNodeName().equals("element") && !rootElement.getNodeName().equals("module")) {
			throw new ElementDescriptionException("Root element must be either 'element' or 'module'.");
		}
		
		ElementDescription eld=new ElementDescription();

		eld.make=new java.util.LinkedList();

		if (!rootElement.hasAttribute("name")) {
			throw new ElementDescriptionException("Element doesn't have 'name' attribute.");
		}
		eld.name=rootElement.getAttribute("name");

		if (!rootElement.hasAttribute("type") && rootElement.getNodeName().equals("element")) {
			throw new ElementDescriptionException("Element doesn't have 'type' attribute.");
		}
		eld.type=rootElement.getAttribute("type");

		if (!rootElement.hasAttribute("mangled")) {
			throw new ElementDescriptionException("Element doesn't have 'mangled' attribute.");
		}
		eld.mangled=rootElement.getAttribute("mangled");

		java.util.LinkedList condProps=new java.util.LinkedList();
		Node nodeL1, nodeL2, nodeL3;
		nodeL1=rootElement.getFirstChild();
		while (nodeL1!=null) {
			if (nodeL1.getNodeType()==Node.ELEMENT_NODE && nodeL1.getNodeName().equals("make")) {
				nodeL2=nodeL1.getFirstChild();
				while (nodeL2!=null) {
					if (nodeL2.getNodeType()==Node.ELEMENT_NODE && nodeL2.getNodeName().equals("command")) {
						Element el=(Element)nodeL2;
						ElementDescriptionCommand cmd=new ElementDescriptionCommand();
						if (el.hasAttribute("action")) {
							cmd.action=el.getAttribute("action");
						} else {
							throw new ElementDescriptionException("Command element must contain attribute 'action'.");
						}
						
						java.util.LinkedList cmdParams=new java.util.LinkedList();
						nodeL3=nodeL2.getFirstChild();
						while (nodeL3!=null) {
							if (nodeL3.getNodeType()==Node.ELEMENT_NODE && nodeL3.getNodeName().equals("param")) {
								Element el2=(Element)nodeL3;
								cmdParams.add(new SOFA.Connector.Property(el2.getAttribute("name"),el2.getAttribute("value")));
							}
							nodeL3=nodeL3.getNextSibling();
						}

						cmd.params=new SOFA.Connector.Property[cmdParams.size()];
						java.util.Iterator iter=cmdParams.iterator();
						for (int i=0;iter.hasNext();i++) {
							cmd.params[i]=(SOFA.Connector.Property)iter.next();
						}
						eld.make.add(cmd);
					} else if (nodeL2.getNodeType()==Node.ELEMENT_NODE && nodeL2.getNodeName().equals("build")) {
						Element el=(Element)nodeL2;
						ElementDescriptionBuild bld=new ElementDescriptionBuild();
						if (el.hasAttribute("element")) {
							bld.element=el.getAttribute("element");
						} else {
							throw new ElementDescriptionException("Build element must contain attribute 'element'.");
						}
						
						if (el.hasAttribute("pathVar")) {
							bld.pathVar=el.getAttribute("pathVar");
						}
						
						if (el.hasAttribute("packageVar")) {
							bld.packageVar=el.getAttribute("packageVar");
						}
						
						java.util.LinkedList bldProps=new java.util.LinkedList();
						nodeL3=nodeL2.getFirstChild();
						while (nodeL3!=null) {
							if (nodeL3.getNodeType()==Node.ELEMENT_NODE && nodeL3.getNodeName().equals("property")) {
								Element el2=(Element)nodeL3;
								bldProps.add(new SOFA.Connector.Property(el2.getAttribute("name"),el2.getAttribute("value")));
							}
							nodeL3=nodeL3.getNextSibling();
						}

						bld.props=new SOFA.Connector.Property[bldProps.size()];
						java.util.Iterator iter=bldProps.iterator();
						for (int i=0;iter.hasNext();i++) {
							bld.props[i]=(SOFA.Connector.Property)iter.next();
						}
						eld.make.add(bld);
					}
					nodeL2=nodeL2.getNextSibling();
				}
			} else if (nodeL1.getNodeType()==Node.ELEMENT_NODE && nodeL1.getNodeName().equals("conditions")) {
				nodeL2=nodeL1.getFirstChild();
				while (nodeL2!=null) {
					if (nodeL2.getNodeType()==Node.ELEMENT_NODE && nodeL2.getNodeName().equals("property")) {
						Element el2=(Element)nodeL2;
						condProps.add(new SOFA.Connector.Property(el2.getAttribute("name"),el2.getAttribute("value")));
					}
					nodeL2=nodeL2.getNextSibling();
				}
			}
			nodeL1=nodeL1.getNextSibling();
		}

		eld.conditions=new SOFA.Connector.Property[condProps.size()];
		java.util.Iterator iter=condProps.iterator();
		for (int i=0;iter.hasNext();i++) {
			eld.conditions[i]=(SOFA.Connector.Property)iter.next();
		}
		
		return eld;
	}

	static public ElementDescription fromXMLFile(String file) throws ElementDescriptionException {
		try {
			javax.xml.parsers.DocumentBuilderFactory documentBuilderFactory=javax.xml.parsers.DocumentBuilderFactory.newInstance();
			javax.xml.parsers.DocumentBuilder documentBuilder=documentBuilderFactory.newDocumentBuilder();
			Document configDoc=documentBuilder.parse(new java.io.File(file));
			return fromXML(configDoc.getDocumentElement());
		} catch (Exception e) {
			throw new ElementDescriptionException("Can't load element from file '"+file+"'.",e);
		}
	}
}
