/* $Id: CompExprBinOper.java,v 2.1 2002/05/15 14:47:34 hnetynka Exp $ */
package SOFA.SOFAnode.Made.CDL;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.RemoteException;

class CompExprBinOper extends CompExprOper {
  public CompExprOper operand1;
  public CompExprOper operand2;
  public CompExprBinOper(int type, CompExprOper op1, CompExprOper op2) {
    super(type);
    operand1 = op1;
    operand2 = op2;
  }

  public int type() { 
    if (operand1.type()==CompExprKind.e_all) 
      return operand2.type();
    else 
      return operand1.type();
  }

  public boolean isProp() { return (operand1.isProp() || operand2.isProp()); }

  public EnumList nameProp(CompRepository rep) throws CDLExceptRemote, CDLExceptLock {
    EnumList l1 = operand1.nameProp(rep);
    EnumList l2 = operand2.nameProp(rep);
    l2.toFirst();
    for (int i=0;i<l2.size();i++) {
      l1.addName((String) l2.aktual());
      l2.toNext();
    }
    return l1;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer("(");
    sb.append(operand1.toString());
    switch (kind) {
    case CompExprKind.e_or:
      sb.append("|");
      break;
    case CompExprKind.e_xor:
      sb.append("^");
      break;
    case CompExprKind.e_and:
      sb.append("&");
      break;
    case CompExprKind.e_shr:
      sb.append(">>");
      break;
    case CompExprKind.e_shl:
      sb.append("<<");
      break;
    case CompExprKind.e_plus:
      sb.append("+");
      break;
    case CompExprKind.e_minus:
      sb.append("-");
      break;
    case CompExprKind.e_mul:
      sb.append("*");
      break;
    case CompExprKind.e_div:
      sb.append("/");
      break;
    case CompExprKind.e_mod:
      sb.append("%");
      break;
    }
    sb.append(operand2.toString());
    sb.append(")");
    return sb.toString();
  }

  public ExprOperDef toNormal(CDLRepository rep) throws CDLExceptToNormal, CDLExceptRemote, CDLExceptLock {
    try {
      return ((Repository) rep.get_containing_repository()).create_exprbinoperation(kind, operand1.toNormal(rep), operand2.toNormal(rep));
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (TIRExceptCreate ecr) {
      throw new CDLExceptToNormal("Can't create expression");
    }
  }

  public static boolean isCompatible(int operand, CompExprOper e1, CompExprOper e2) {
    switch (operand) {
      case CompExprKind.e_mul:
      case CompExprKind.e_div:
      case CompExprKind.e_plus:
      case CompExprKind.e_minus:
        switch (e1.type()) {
          case CompExprKind.e_int:
            if ((e2.type() == CompExprKind.e_all) || (e2.type() == CompExprKind.e_int))
              return true;
            break;
          case CompExprKind.e_float:
            if ((e2.type() == CompExprKind.e_all) || (e2.type() == CompExprKind.e_float))
              return true;
            break;
          case CompExprKind.e_fixed:
            if ((e2.type() == CompExprKind.e_all) || (e2.type() == CompExprKind.e_fixed))
              return true;
            break;
          case CompExprKind.e_all:
            if ((e2.type() == CompExprKind.e_all) || (e2.type() == CompExprKind.e_fixed) ||
                (e2.type() == CompExprKind.e_int) || (e2.type() == CompExprKind.e_float))
              return true;
            break;
        }
        return false;
      case CompExprKind.e_and:
      case CompExprKind.e_xor:
      case CompExprKind.e_or:
      case CompExprKind.e_shl:
      case CompExprKind.e_shr:
      case CompExprKind.e_mod:
        switch (e1.type()) {
          case CompExprKind.e_int:
            if ((e2.type() == CompExprKind.e_all) || (e2.type() == CompExprKind.e_int))
              return true;
            break;
          case CompExprKind.e_all:
            if ((e2.type() == CompExprKind.e_all) || (e2.type() == CompExprKind.e_int))
              return true;
            break;
        }
        return false;
      default: return false;
    }
  }
}
