/* $Id: CompExprConstant.java,v 2.1 2002/05/15 14:47:34 hnetynka Exp $ */
package SOFA.SOFAnode.Made.CDL; 
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.RemoteException;

class CompExprConstant extends CompExprOper {
  public CompReffer value;
  
  public CompExprConstant(CompReffer val) {
    super(CompExprKind.e_constant);
    value = val;
  }

  public int type() {
    switch (value.basereffer) {
    case ObjectsKind.o_Primitive:
      switch (value.baserefferprim) {
      case CompPrimKind.p_short:
      case CompPrimKind.p_long:
      case CompPrimKind.p_longlong:
      case CompPrimKind.p_ushort:
      case CompPrimKind.p_ulong:
      case CompPrimKind.p_ulonglong:
        return CompExprKind.e_int;
      case CompPrimKind.p_float:
      case CompPrimKind.p_double:
      case CompPrimKind.p_longdouble:
        return CompExprKind.e_float;
      case CompPrimKind.p_boolean:
        return CompExprKind.e_boolean;
      case CompPrimKind.p_char:
        return CompExprKind.e_char;
      case CompPrimKind.p_wchar:
        return CompExprKind.e_wchar;
      case CompPrimKind.p_string:
        return CompExprKind.e_string;
      case CompPrimKind.p_wstring:
        return CompExprKind.e_wstring;
      }
      case ObjectsKind.o_Enum:
        return CompExprKind.e_enum;
      case ObjectsKind.o_String:
        return CompExprKind.e_string;
      case ObjectsKind.o_Wstring:
        return CompExprKind.e_wstring;
      case ObjectsKind.o_Fixed:
        return CompExprKind.e_fixed;
    }
    return CompExprKind.e_all;
  }

  public String toString() {
    return value.ref.toStringWithoutVersion();
  }

  public EnumList nameProp(CompRepository rep) throws CDLExceptRemote, CDLExceptLock {
    int i,j;
    if (value.isNew) {
      CompContainer con = rep;
      value.ref.sn.names.toFirst();
      for(i=0;i<value.ref.sn.names.size();i++) {
        CompContained[] a = con.lookup((String) value.ref.sn.names.aktual());
        con = (CompContainer) a[0];
        value.ref.sn.names.toNext();
      }  
      CompContained sec = con.lookup(value.ref.name, value.ref.version);
      return ((CompConstant) sec).expr.nameProp(rep);
    } else {
      try {
        Container con = rep.cdlRepository;
        value.ref.sn.names.toFirst();
        for(i=0;i<value.ref.sn.names.size();i++) {
          Contained[] a = con.lookup_name((String) value.ref.sn.names.aktual());
          if (a.length==1) {
            con = (Container) a[0];
          } else {
            for(j=0;j<a.length;j++) {
              if (a[j].get_identification().version().compareTo(value.ref.version)==0) {
                con = (Container) a[j];
                break;
              }
            }
          }
          value.ref.sn.names.toNext();
        }
        Contained sec = con.lookup(value.ref.name, value.ref.version);
        return CompExprOper.scanNormalExpr(((ConstantDef) sec).value());
      } catch (TIRExceptLock e) {
        throw new CDLExceptLock("Repository is locked.");
      } catch (RemoteException e) {
        throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
      }
    }
  }

  public ExprOperDef toNormal(CDLRepository rep) throws CDLExceptToNormal, CDLExceptRemote, CDLExceptLock {
    try {
      int i,j;
      Container con = rep;
      value.ref.sn.names.toFirst();
      for(i=0;i<value.ref.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) value.ref.sn.names.aktual());
        if (a.length==1) {
          con = (Container) a[0];
        } else {
          for(j=0;j<a.length;j++) {
            if (a[j].get_identification().version().compareTo(value.ref.version)==0) {
              con = (Container) a[j];
              break;
            }
          }
        }
        value.ref.sn.names.toNext();
      }
      Contained sec = con.lookup(value.ref.name, value.ref.version);
      return ((Repository) rep.get_containing_repository()).create_exprconstant((ConstantDef) sec);
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (TIRExceptCreate ecr) {
      throw new CDLExceptToNormal("Can't create expression");
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    }
  }
  
  public boolean isProp() { return false; /* prepsat !!! */ }
 
  public void checkConsist(EnumList props, CompRepository rep) throws CDLExceptCheck, CDLExceptLock, CDLExceptRemote {
    value.checkConsist(props, rep);
  }

 /* public int isPositive() {
    if (typeOfConst == CompExprKind.e_int) {
      return -1;   //!!! dodelat
    } else 
      return -2;
  }*/

}
