/* $Id: CompRepository.java,v 2.1 2002/04/17 13:41:40 hnetynka Exp $ */
package SOFA.SOFAnode.Made.CDL;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.io.*;

class CompRepository extends CompContainerIm {
  class CompRepExcept extends Exception {
    public CompRepExcept() { super(); }
    public CompRepExcept(String s) { super(s); }
  }
  
  public Repository defRepository;
  public WorkRepository workRepository;
  public CDLRepository cdlRepository;    // work CDLRepository
  public ProfileDef profile;
  
  public Scopename aktScope;

  public List defaultTypes;

  public String searchError;

  public boolean useProtocols;     // save protocols, protocols must exist
  public boolean testProtocols;    // test protocols --- Stano (testProtocols implies useProtocols

  public SOFA.SOFAnode.Util.BehaviorProtocolChecker protocolChecker;
  
  public CompRepository(Scopename aktScope) throws Exception {
    super(ObjectsKind.o_Repository);
    this.aktScope = aktScope;

    String rmiport = System.getProperty("tir.rmiport","1099");
    String rmihost = System.getProperty("tir.rmihost","localhost");
    defRepository = (Repository) Naming.lookup("//"+rmihost+":"+rmiport+"/Repository");
    
    boolean ok = false;
    int i = 0;
    
    String profName = System.getProperty("cdl.profile",null);
    while (!ok) {
      try {
        if (profName==null) 
          profile = null;
        else {
          profile = defRepository.getProfiles().lookup(profName);
          if (profile == null)
            throw new CDLExceptBadName("Profile with name "+profName+" doesn't exist.");
        }
        ok = true;
      } catch (TIRExceptLock e) {
        if (i == Consts.repeatCount)
          throw new CDLExceptLock("Repository is locked.");
        else {
          i++;
          System.err.println("Repository is locked. Waiting for next attempt.");
          Thread.sleep(Consts.repeatDelay);
        }  
      }
    }
    
    ok = false;
    i = 0;
    while (!ok) {
      try {
        if (System.getProperty("cdl.debug","no").compareTo("yes")==0) {
          System.out.println(java.util.Calendar.getInstance().getTime().getTime());
          workRepository = defRepository.beginChanges(profile);
          System.out.println(java.util.Calendar.getInstance().getTime().getTime());
        } else {
          workRepository = defRepository.beginChanges(profile);
        }
        profile = workRepository.profile();
        ok = true;
      } catch (TIRExceptLock e) {
        if (i == Consts.repeatCount)
          throw new CDLExceptLock("Repository is locked.");
        else {
          i++;
          System.err.println("Repository is locked. Waiting for next attempt.");
          Thread.sleep(Consts.repeatDelay);
        }  
      }
    }

    cdlRepository = (CDLRepository) workRepository.lookup("cdl","");
   
    useProtocols = false;
    testProtocols = false;
    String prot = System.getProperty("cdl.protocols","no");
    if (prot.compareTo("yes")==0)
      useProtocols = true;
    prot = System.getProperty("cdl.testprotocols","no");
    if (prot.compareTo("yes")==0) {
      useProtocols = true;
      testProtocols = true;
    }
    
    defaultTypes = new List();
    searchError = null;

    protocolChecker = SOFA.SOFAnode.Util.BehaviorProtocolCheckerFactory.getChecker(); 
  }

  /** for compiler launched from external programs (Netbeans) */
  public CompRepository(Scopename aktScope, Repository defRep, WorkRepository workRep, boolean useProt, boolean testProt) throws Exception {
    super(ObjectsKind.o_Repository);
    this.aktScope = aktScope;

    defRepository = defRep;
    
    workRepository = workRep;
    profile = workRepository.profile();
    cdlRepository = (CDLRepository) workRepository.lookup("cdl","");
   
    useProtocols = useProt;
    testProtocols = testProt;
    if (testProt) {
      useProtocols = true;
      System.setProperty("cdl.protocols","yes");
      System.setProperty("cdl.testprotocols","yes");
    } else {
      System.setProperty("cdl.testprotocols","no");
      if (useProtocols)
        System.setProperty("cdl.protocols","yes");
      else
        System.setProperty("cdl.protocols","no");
    }
    
    defaultTypes = new List();
    searchError = null;

    protocolChecker = SOFA.SOFAnode.Util.BehaviorProtocolCheckerFactory.getChecker(); 
  }

  public void checkConsist(EnumList props, CompRepository rep) throws CDLExceptCheck, CDLExceptLock, CDLExceptRemote {
    CompContained[] cont = contents();
    for (int i=0;i<cont.length;i++) {
      switch (cont[i].objectKind()) {
      case ObjectsKind.o_Module:
        cont[i].checkConsist(null,rep);
        break;
      case ObjectsKind.o_Interface:
        cont[i].checkConsist(null,rep);
        break;
      case ObjectsKind.o_Frame:
        try {
          cont[i].checkConsist(null,rep);
        } catch (CDLExceptCheck e) {
          throw new CDLExceptCheck(e.getMessage()+" in frame "+cont[i].fullName());
        }
        break;
      case ObjectsKind.o_Provider:
        cont[i].checkConsist(null,rep);
      }
    }
  }

/*  private void readDefaults() throws CDLExceptLock, CDLExceptRemote {
    try {
      String fileName = System.getProperty("cdl.defaultfile","cdldef");
      BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
      String str;
      int lineCnt = 0;
      while ((str = in.readLine())!=null) {
        lineCnt++;
        str = str.trim();     // remove white spaces from begin and end
        if (str.length()==0)  // empty line
          continue;
        if (str.charAt(0)=='#')  // full line coment
          continue;
        int i;
        if ((i = str.indexOf('#'))!=-1) {  // comment in line
          str = str.substring(0,i);
          str = str.trim();
        }
        if (str.indexOf(' ')==-1) {
          System.err.println("Error in file "+fileName+" at line "+lineCnt+".");
          continue;
        }  
        StringBuffer name = new StringBuffer();
        StringBuffer version = new StringBuffer();
        i = 0;
        while (str.charAt(i)!=' ') {
          name.append(str.charAt(i));
          i++;
        }
        while (str.charAt(i)==' ')
          i++;
        while (str.length() > i || str.charAt(i)!=' ') {
          version.append(str.charAt(i));
          i++;
        }
        if (name.charAt(0)!=':') {
          System.err.println("You must set absolute name (line: "+lineCnt+").");
          continue;
        }
        DefaultTypes t = new DefaultTypes(name.toString(),version.toString());
        boolean exist = false;
        boolean cycle = true;
        int c = 0;
        while (cycle) {
          try {
            exist = t.test(defRepository);
            cycle = false;
          } catch (CDLExceptLock e) {
            try {
              Thread.sleep(2000);
            } catch (InterruptedException e1) {;}  
            c++;
            if (c == 10)
              throw new CDLExceptLock("Repository is locked.");
          }
        }  
        if (exist)
          defaultTypes.add(t);
        else 
          System.err.println("Object at line "+lineCnt+" in "+fileName+" doesn't exist.");
      }
    } catch (IOException e) {
      System.err.println("Error when reading default types: "+e.getMessage());
    }
  } */

  public CompReffer searchReffer(String name, String version) throws CDLExceptLock, CDLExceptRemote {
    if (name!=null && name.length()!=0) {
      // convert name&ver to FullID
      FullID id = null;
      if (name.charAt(0)==':') {
        if (aktScope.length()==0)
          id = new FullID ("::"+name , version);
        else
          id = new FullID (aktScope.toString()+"::"+name , version);
      } else {
        id = new FullID (name, version);
      }
      return searchReffer(id);
    } else
      return null;  
  }

/*  static boolean isContainer(int kind) {
    if (kind == ObjectsKind.o_Module) return true;
    if (kind == ObjectsKind.o_AModule) return true;
    if (kind == ObjectsKind.o_Provider) return true;
    if (kind == ObjectsKind.o_Interface) return true;
    if (kind == ObjectsKind.o_Struct) return true;
    if (kind == ObjectsKind.o_Union) return true;
    if (kind == ObjectsKind.o_Architecture) return true;
    if (kind == ObjectsKind.o_Exception) return true;
    if (kind == ObjectsKind.o_Frame) return true;
    return false;   
  }*/

/*  static boolean isDefContainer(int kind) {
    if (kind == DefinitionKind.dk_Module) return true;
    if (kind == DefinitionKind.dk_AModule) return true;
    if (kind == DefinitionKind.dk_Provider) return true;
    if (kind == DefinitionKind.dk_Interface) return true;
    if (kind == DefinitionKind.dk_Struct) return true;
    if (kind == DefinitionKind.dk_Union) return true;
    if (kind == DefinitionKind.dk_Architecture) return true;
    if (kind == DefinitionKind.dk_Exception) return true;
    if (kind == DefinitionKind.dk_Frame) return true;
    return false;   
  }*/

  public CompReffer searchReffer(FullID id) throws CDLExceptLock, CDLExceptRemote {
    return searchReffer(id, 0);
  }

  /** .
    * @param cancelNameSpace   how many namespace have to be skiped in searching
    */
  public CompReffer searchReffer(FullID id, int cancelNameSpace) throws CDLExceptLock, CDLExceptRemote {
    int i,j;
    long numPath = 0;
//  long tmb = java.util.Calendar.getInstance().getTime().getTime();
    if (!id.isAbsolute) {
      // set absolute path to "id"
      FullID nid = new FullID(true);
      aktScope.names.toFirst();
      for(i=0;i<aktScope.length()-cancelNameSpace;i++) {
        nid.sn.addScope((String)aktScope.names.aktual());
	aktScope.names.toNext();
      }
      numPath = aktScope.length();
      id.sn.names.toFirst();
      for(i=0;i<id.sn.length();i++) {
        nid.sn.addScope((String)id.sn.names.aktual());
	id.sn.names.toNext();
      }
      nid.name = id.name;
      nid.setVersion(id.version);
      id = nid;
    }
//  long tme = java.util.Calendar.getInstance().getTime().getTime();
//  Output.out.println("#####"+Long.toString(tme-tmb)+" ms "+id.toString());
    
    switch (id.isin) {
      case IDKind.none : return searchRefferDefault(id, numPath);
      case IDKind.version : return searchRefferVersion(id, numPath);
      case IDKind.tag : return searchRefferTag(id, numPath);
    }
    return null;
  }

  private CompReffer searchRefferVersion(FullID id, long numPath) throws CDLExceptLock, CDLExceptRemote {
    int i,j;

    try {
       // searching in new objects
    CompContainer con = this;
    id.sn.names.toFirst();
    for(i=0;i<id.sn.names.size();i++) {
      CompContained[] a = con.lookup((String) id.sn.names.aktual());
      if (a==null || a.length==0) 
        throw new CompRepExcept();
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else throw new CompRepExcept();
      } else {
        boolean found = false;
        for(j=0;j<a.length;j++) {
          if (a[j].getIdent().version.compareTo(id.version)==0) {
            if ( ! a[j].isContainer()) throw new CompRepExcept();
            found = true;
            con = (CompContainer) a[j];
            break;
          }
        }
        if (!found) throw new CompRepExcept();
      }
      id.sn.names.toNext();
    }
    CompContained sec = con.lookup(id.name, id.version);
    if (sec!=null) {
      CompReffer ret = new CompReffer();
      ret.what = sec.objectKind();
      ret.ref = new FullID(true);
      ret.ref.name = new String(id.name);
      ret.ref.version = new String(id.version);
      ret.ref.isin = IDKind.version;
      ret.isNew = true;
      id.sn.names.toFirst();
      for(j=0;j<id.sn.names.size();j++) {
        ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
        id.sn.names.toNext();
      }
      if (ret.what == ObjectsKind.o_Frame)
        ret.isSystem = ((CompFrame)sec).isSystem;
      if (ret.what == ObjectsKind.o_Architecture)
        ret.isSystem = ((CompArchitecture)sec).isSystem;
      return ret;
    } 
       // end of searching in new objects
    } catch (CompRepExcept e) {;}
    
       // search in normal repository
    try {
      Container con = cdlRepository;
      id.sn.names.toFirst();
      for(i=0;i<id.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) id.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          //if (isDefContainer(a[0].get_def_kind().value())) con = (Container) a[0];
          if (a[0] instanceof Container) con = (Container) a[0];
          else throw new CompRepExcept();
        } else {
          boolean found = false;
          for(j=0;j<a.length;j++) {
            if (a[j].get_identification().version().compareTo(id.version)==0) {
              if ( !(a[j] instanceof Container)) throw new CompRepExcept();
              found = true;
              con = (Container) a[j];
              break;
            }
          }
          if (!found) throw new CompRepExcept();
        }
        id.sn.names.toNext();
      }
      Contained sec = con.lookup(id.name, id.version);
      if (sec!=null) {
        CompReffer ret = new CompReffer();
        ret.what = sec.get_def_kind().value();
        ret.ref = new FullID(true);
        ret.ref.name = new String(id.name);
        ret.ref.version = new String(id.version);
        ret.ref.isin = IDKind.version;
        ret.isNew = false;
        id.sn.names.toFirst();
        for(j=0;j<id.sn.names.size();j++) {
          ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
          id.sn.names.toNext();
        }
	if (ret.what == DefinitionKind.dk_Frame)
	  ret.isSystem = ((FrameDef)sec).is_system();
	if (ret.what == DefinitionKind.dk_Architecture)
	  ret.isSystem = ((ArchitectureDef)sec).is_system();
        return ret;
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}
      // end of search in normal repository
    
      // search in Template Repository
      // currently no interfaces to TR

    // search in higher name scope (if possible)
    if (numPath > 0) {
      id.sn.names.toFirst();
      for(i=1; i<numPath; i++)
        id.sn.names.toNext();
      id.sn.names.removeAkt();
      return searchRefferVersion(id, numPath-1);
    } else {
      searchError = id.name + ":" + id.version + " doesn't exist";
      return null;
    }
  }

  private CompReffer searchRefferTag(FullID id, long numPath) throws CDLExceptLock, CDLExceptRemote {
    int i,j;

    try {
       // searching in new objects
    CompContainer con = this;
    id.sn.names.toFirst();
    for(i=0;i<id.sn.names.size();i++) {
      CompContained[] a = con.lookup((String) id.sn.names.aktual());
      if (a==null || a.length==0) 
        throw new CompRepExcept();
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else throw new CompRepExcept();
      } else {
        throw new CompRepExcept();
      }
      id.sn.names.toNext();
    }
    CompContained sec = con.lookupTag(id.name, id.tag);
    if (sec!=null) {
      CompReffer ret = new CompReffer();
      ret.what = sec.objectKind();
      ret.ref = new FullID(true);
      ret.ref.name = new String(id.name);
      ret.ref.version = new String(sec.getIdent().version);
      ret.ref.isin = IDKind.version;
      ret.isNew = true;
      id.sn.names.toFirst();
      for(j=0;j<id.sn.names.size();j++) {
        ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
        id.sn.names.toNext();
      }
      if (ret.what == ObjectsKind.o_Frame)
        ret.isSystem = ((CompFrame)sec).isSystem;
      if (ret.what == ObjectsKind.o_Architecture)
        ret.isSystem = ((CompArchitecture)sec).isSystem;
      return ret;
    } 
       // end of searching in new objects
    } catch (CompRepExcept e) {;}
    
       // search in normal repository
    try {
      Container con = cdlRepository;
      id.sn.names.toFirst();
      for(i=0;i<id.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) id.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else throw new CompRepExcept();
        } else {
          throw new CompRepExcept();
        }
        id.sn.names.toNext();
      }
      Contained sec = con.lookup_tag(id.name, id.tag);
      if (sec!=null) {
        CompReffer ret = new CompReffer();
        ret.what = sec.get_def_kind().value();
        ret.ref = new FullID(true);
        ret.ref.name = new String(id.name);
        ret.ref.version = new String(sec.get_identification().version());
        ret.ref.isin = IDKind.version;
        ret.isNew = false;
        id.sn.names.toFirst();
        for(j=0;j<id.sn.names.size();j++) {
          ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
          id.sn.names.toNext();
        }
	if (ret.what == DefinitionKind.dk_Frame)
	  ret.isSystem = ((FrameDef)sec).is_system();
	if (ret.what == DefinitionKind.dk_Architecture)
	  ret.isSystem = ((ArchitectureDef)sec).is_system();
        return ret;
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}
      // end of search in normal repository
    
      // search in Template Repository
      // currently no interfaces to TR
    // search in higher name scope (if possible)
    if (numPath > 0) {
      id.sn.names.toFirst();
      for(i=1; i<numPath; i++)
        id.sn.names.toNext();
      id.sn.names.removeAkt();
      return searchRefferTag(id, numPath-1);
    } else {
      searchError = id.name + ":" + id.version + " doesn't exist";
      return null;
    }
  }

  private CompReffer searchRefferDefault(FullID id, long numPath) throws CDLExceptLock, CDLExceptRemote {
    // first search in new objects
    int i,j;

//  long tmb = java.util.Calendar.getInstance().getTime().getTime();
    try {

    CompContainer con = this;
    id.sn.names.toFirst();
    for(i=0;i<id.sn.names.size();i++) {
      CompContained[] a = con.lookup((String) id.sn.names.aktual());
      if (a==null || a.length==0) 
        throw new CompRepExcept();
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else throw new CompRepExcept();
      } else {
        throw new CompRepExcept();
      }
      id.sn.names.toNext();
    }
    CompContained[] sec = con.lookup(id.name);
    if (sec!=null && sec.length!=0) {
      if (sec.length==1) {
        CompReffer ret = new CompReffer();
        ret.what = sec[0].objectKind();
        ret.ref = new FullID(true);
        ret.ref.name = new String(id.name);
        ret.ref.version = new String(sec[0].getIdent().version);
        ret.ref.isin = IDKind.version;
        ret.isNew = true;
        id.sn.names.toFirst();
        for(j=0;j<id.sn.names.size();j++) {
          ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
          id.sn.names.toNext();
        }
        if (ret.what == ObjectsKind.o_Frame)
          ret.isSystem = ((CompFrame)sec[0]).isSystem;
        if (ret.what == ObjectsKind.o_Architecture)
          ret.isSystem = ((CompArchitecture)sec[0]).isSystem;
         
        return ret;
      } else {
        searchError = "it never should be here";
        return null;
      }
    } 
       // end of searching in new objects
    } catch (CompRepExcept e) {;}
    
    // searching in profiles
    try {
      if (profile != null) {
        Identification idl = profile.lookup(id.langToString());
        if (idl != null) {
          CompReffer ret = new CompReffer();
          ret.ref = new FullID(true);
          ret.ref.name = new String(idl.name());
          ret.ref.version = new String(idl.version());
          ret.ref.isin = IDKind.version;
          ret.isNew = false;
          id.sn.names.toFirst();
          for(j=0;j<id.sn.names.size();j++) {
            ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
            id.sn.names.toNext();
          }
          try {   // searching object with idl in identification for obtaining its kind
            Container con = cdlRepository;
            id.sn.names.toFirst();
            for(i=0;i<id.sn.names.size();i++) {
              Contained[] a = con.lookup_name((String) id.sn.names.aktual());
              if (a==null || a.length==0) {
                throw new CompRepExcept();
              }  
              if (a.length==1) {
                if (a[0] instanceof Container) con = (Container) a[0];
                else {
                  throw new CompRepExcept();
                }  
              } else {
                boolean found = false;
                for(j=0;j<a.length;j++) {
                  if (a[j].get_identification().version().compareTo(idl.version())==0) {
                    if ( !(a[j] instanceof Container)) {
                      throw new CompRepExcept();
                    }
                    found = true;
                    con = (Container) a[j];
                    break;
                  }
                }
                if (!found) {
                  throw new CompRepExcept();
                }
              }
              id.sn.names.toNext();
            }
            Contained sec = con.lookup_lastinbranch(id.name, "");
            if (sec!=null) {
              ret.what = sec.get_def_kind().value();
            }
            if (ret.what == DefinitionKind.dk_Frame)
	      ret.isSystem = ((FrameDef)sec).is_system();
	    if (ret.what == DefinitionKind.dk_Architecture)
	      ret.isSystem = ((ArchitectureDef)sec).is_system();
          } catch (TIRExceptLock e) {
            throw new CDLExceptLock("Repository is locked.");
          } catch (RemoteException e) {
            throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
          } catch (CompRepExcept e) {
	    searchError = "error in profile";
            return null; 
          }
          return ret;
        }
      }
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } 
    // end of search in profiles

    // search for head version in normal repository
    try {
      Container con = cdlRepository;
      id.sn.names.toFirst();
      for(i=0;i<id.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) id.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else throw new CompRepExcept();
        } else {
          Contained last = con.lookup_lastinbranch((String) id.sn.names.aktual(),"");
          if (last == null)
            throw new CompRepExcept();
          else
            con = (Container) last;
          /*boolean found = false;
          for(j=0;j<a.length;j++) {
            if (a[j].get_identification().version().compareTo(id.version)==0) {
              if ( !(a[j] instanceof Container)) throw new CompRepExcept();
              found = true;
              con = (Container) a[j];
              break;
            }
          }
          if (!found) throw new CompRepExcept();*/
        }
        id.sn.names.toNext();
      }
      Contained sec = con.lookup_lastinbranch(id.name, "");
      if (sec!=null) {
        CompReffer ret = new CompReffer();
        ret.what = sec.get_def_kind().value();
        ret.ref = new FullID(true);
        ret.ref.name = new String(id.name);
        ret.ref.version = new String(sec.get_identification().version());
        ret.ref.isin = IDKind.version;
        ret.isNew = false;
        id.sn.names.toFirst();
        for(j=0;j<id.sn.names.size();j++) {
          ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
          id.sn.names.toNext();
        }
	if (ret.what == DefinitionKind.dk_Frame)
	  ret.isSystem = ((FrameDef)sec).is_system();
	if (ret.what == DefinitionKind.dk_Architecture)
	  ret.isSystem = ((ArchitectureDef)sec).is_system();
        return ret;
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}

    // search in higher name scope (if possible)
    if (numPath > 0) {
      id.sn.names.toFirst();
      for(i=1; i<numPath; i++)
        id.sn.names.toNext();
      id.sn.names.removeAkt();

      CompReffer ret = searchRefferDefault(id, numPath-1);
  
//  long tme = java.util.Calendar.getInstance().getTime().getTime();
//  Output.out.println("@@@@@"+Long.toString(tme-tmb)+" ms "+id.toString());
         
      return ret;
    } else {
      searchError = id.name + ":" + id.version + " doesn't exist";
      return null;
    }
  }

  public CompModule searchExistNewMod(String name) {
    Scopename fullname = new Scopename();
    int i;
    aktScope.names.toFirst();
    for (i=0;i<aktScope.length();i++) {
      fullname.addScope((String)aktScope.names.aktual());
      aktScope.names.toNext();
    }
    fullname.addScope(name);

    CompContainer con = this;
    fullname.names.toFirst();
    for(i=0;i<fullname.names.size()-1;i++) {
      CompContained[] a = con.lookup((String) fullname.names.aktual());
      if (a==null || a.length==0) 
        return null;
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else {
          return null;
        }  
      } else {
        return null;
      }
      fullname.names.toNext();
    }
    CompContained sec[] = con.lookup((String) fullname.names.aktual());
    if (sec!=null && sec.length!=0) {
      if (sec[0].objectKind() == ObjectsKind.o_Module) {
        return (CompModule) sec[0];
      } else {
        return null;
      }
    }
    return null;
  }

  // name is short!
  public ID searchForNewMod(String name) throws CDLExceptLock, CDLExceptRemote {
    searchError = null;
    Scopename fullname = new Scopename();
    int i;
    aktScope.names.toFirst();
    for (i=0;i<aktScope.length();i++) {
      fullname.addScope((String)aktScope.names.aktual());
      aktScope.names.toNext();
    }
    fullname.addScope(name);

    try {
       // searching in new objects
      CompContainer con = this;
      fullname.names.toFirst();
      for(i=0;i<fullname.names.size()-1;i++) {
        CompContained[] a = con.lookup((String) fullname.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0].isContainer()) con = (CompContainer) a[0];
          else {
            searchError = "This absolute name can not be module";       
            return null;
          }  
        } else {
          searchError = "This absolute name can not be module";       
          return null;
        }
        fullname.names.toNext();
      }
      CompContained sec[] = con.lookup((String) fullname.names.aktual());
      if (sec!=null && sec.length!=0) {
        if (sec[0].objectKind() != ObjectsKind.o_Module) {
          searchError = "Object with this name and other kind exists";
          return null;
        } else {  // never should be here (searchExistNewMod())
          return null;
        }
      } else {   // here will be scaned enums
        sec = con.contents();
        for (i=0;i<sec.length;i++) {   // scan enums 
          if (sec[i].objectKind()==ObjectsKind.o_Enum) {
            if (((CompEnum) sec[i]).members.isIn((String) fullname.names.aktual())) {
              searchError = "Object with this name and other kind exists";
              return null;
            }
          }
        }
      }
       // end of searching in new objects
    } catch (CompRepExcept e) {;}

      // search in normal repository
    try {
      Container con = cdlRepository;
      fullname.names.toFirst();
      for(i=0;i<fullname.names.size()-1;i++) {
        Contained[] a = con.lookup_name((String) fullname.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else {
            searchError = "This absolute name can not be module";       
            return null;
          }
        } else {
          searchError = "This absolute name can not be module";       
          return null;
        }
        fullname.names.toNext();
      }
      Contained[] sec = con.lookup_name((String) fullname.names.aktual());
      if (sec!=null && sec.length!=0) {
        if (sec[0].get_def_kind().value() == DefinitionKind.dk_Module) {
          ID ret = null;
          try {
            Identification idl = sec[0].get_identification();
            ret = new ID (idl.name(), idl.version());
          } catch (RemoteException e) {
            throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
          }
          ret.exist = true;
          ret.id = null;
          ret.what = ObjectsKind.o_Module;
          return ret;
        } else {
          searchError = "Object with this name exists in the repository.";
          return null;
        }  
      } else {  // here will be scaned enums
        DefinitionKind endk = workRepository.get_spec_def_kind(DefinitionKind.dk_Enum);
        sec = con.contents(endk);
        for (i=0;i<sec.length;i++) {
          EnumMember[] membs = ((EnumDef) sec[i]).members();
          for (int j=0;j<membs.length;j++) {
            if (membs[j].name().compareTo((String) fullname.names.aktual())==0) {
              searchError = "Object with this name exists in the repository.";
              return null;
            }
          }
        }
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}
      // end of search in normal repository

    Identification newVer = null;
    try {
      newVer = workRepository.create_identification(ID.lang, fullname.toString(),"");
    } catch (TIRExceptCreate e) {
      searchError = "Can't create identification";
      return null;
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    ID ret = null;
    try {
      ret = new ID (newVer.name(), newVer.version());
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    ret.id = newVer;
    ret.what = ObjectsKind.o_Module;
    return ret;
  }

  public ID searchForNewName(ID id) throws CDLExceptLock, CDLExceptRemote {
    int i,j;
    // set absolute path to "id"
    FullID nid = new FullID(true);
    aktScope.names.toFirst();
    for(i=0;i<aktScope.length();i++) {
      nid.sn.addScope((String)aktScope.names.aktual());
      aktScope.names.toNext();
    }
    nid.name = id.name;
    nid.setVersion(id.version);
    if (id.tag != null) {
      nid.tag = id.tag;
      nid.isin = IDKind.tag;
    }

    switch (id.isin) {
      case IDKind.none : return searchForNewDefault(nid, id.what);
      case IDKind.tag : return searchForNewTag(nid, id.what);
      case IDKind.version : 
        searchError = "Specify tag or nothing for default.";
        return null;
    }
    return null;
  }

  public ID searchForNewArch(FullID id) throws CDLExceptLock, CDLExceptRemote {
    switch (id.isin) {
      case IDKind.none : return searchForNewDefault(id, ObjectsKind.o_Architecture);
      case IDKind.tag : return searchForNewTag(id, ObjectsKind.o_Architecture);
      case IDKind.version : 
        searchError = "Specify tag or nothing for default.";
        return null;
    }
    return null;
  }

  private ID searchForNewDefault(FullID id, int kind) throws CDLExceptLock, CDLExceptRemote {
    int i,j;

    try {
       // searching in new objects
    CompContainer con = this;
    id.sn.names.toFirst();
    for(i=0;i<id.sn.names.size();i++) {
      CompContained[] a = con.lookup((String) id.sn.names.aktual());
      if (a==null || a.length==0) 
        throw new CompRepExcept();
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else throw new CompRepExcept();
      } else {
        searchError = "It never would be here.";       
        return null;
      }
      id.sn.names.toNext();
    }
    CompContained sec[] = con.lookup(id.name);
    if (sec!=null && sec.length!=0) {
      searchError = "Object with this name was added in this session.";
      return null;
    } else {   // here will be scaned enums
      sec = con.contents();
      for (i=0;i<sec.length;i++) {   // scan enums 
        if (sec[i].objectKind()==ObjectsKind.o_Enum) {
          if (((CompEnum) sec[i]).members.isIn(id.name)) {
            searchError = "Object with this name and other kind exists";
            return null;
          }
        }
      }
    }
       // end of searching in new objects
    } catch (CompRepExcept e) {;}

    // searching in profiles
    try {
      if (profile!=null) {
        Identification idl = profile.lookup(id.langToString());
        if (idl != null) {
          // search for last version from this def. type
          id.sn.names.toFirst();
          try {
            Container con = cdlRepository;
            for(i=0;i<id.sn.names.size();i++) {
              Contained[] a = con.lookup_name((String) id.sn.names.aktual());
              if (a==null || a.length==0) 
                throw new CompRepExcept();
              if (a.length==1) {
                if (a[0] instanceof Container) con = (Container) a[0];
                else throw new CompRepExcept();
              } else {
                boolean found = false;
                for(j=0;j<a.length;j++) {
                  if (a[j].get_identification().version().compareTo(idl.version())==0) {
                    if ( !(a[j] instanceof Container)) throw new CompRepExcept();
                    found = true;
                    con = (Container) a[j];
                    break;
                  }
                }
                if (!found) {
                  searchError = "Error in profiles.";
                  return null;
                }
              }
              id.sn.names.toNext();
            }
            Contained sec = con.lookup(idl.name(), idl.version());
            if (sec==null) {
              searchError = "Error in profiles.";
              return null;
            }
            if (sec.get_def_kind().value() != kind) {
              searchError = "Object with other kind with same name exists in repository.";
              return null;
            }
            Contained lst = con.lookup_lastfromversion(idl.name(), idl.version());
            if (lst.get_identification().is_equal(idl)) {  // next version from this
              Identification newVer = null;
              try {
                newVer = workRepository.create_nextversion(idl);
              } catch (TIRExceptCreate e) {
                searchError = "Can't create next version";
                return null;
              }
              switch (kind) {  // add to the profile only objects of this kinds
              case DefinitionKind.dk_Struct:
              case DefinitionKind.dk_Union:
              case DefinitionKind.dk_Enum:
              case DefinitionKind.dk_Interface:
              case DefinitionKind.dk_Frame:
              case DefinitionKind.dk_Exception:
              case DefinitionKind.dk_Typedef:
              case DefinitionKind.dk_Constant:
              case DefinitionKind.dk_Architecture:
                try {
                  profile.remove(idl.lang_absolute_name().name());
                  profile.add(newVer);
                } catch (TIRExceptCreate e) {
                  searchError = "Error in work with profile";
                  return null;
                }
              }
              ID ret = new ID(newVer.name(), newVer.version());
              ret.id = newVer;
              ret.what = kind;
              return ret;
            } else {  // new branch
              Identification newVer = null;
              try {
                // !!! ?
                Interactive.writeLine("Input branchtag for "+idl.absolute_name().name()+":");
                String br = Interactive.readLine();
                newVer = workRepository.create_branch(idl, br); // !!! jak jinak s tou vetvi
              } catch (TIRExceptCreate e) {
                searchError = "Can't create branch version";
                return null;
              } catch (IOException e) {
                searchError = "IO error";
                return null;
              }
              switch (kind) {  // add to the profile only objects of this kinds
              case DefinitionKind.dk_Struct:
              case DefinitionKind.dk_Union:
              case DefinitionKind.dk_Enum:
              case DefinitionKind.dk_Interface:
              case DefinitionKind.dk_Frame:
              case DefinitionKind.dk_Exception:
              case DefinitionKind.dk_Typedef:
              case DefinitionKind.dk_Constant:
              case DefinitionKind.dk_Architecture:
                try {
                  profile.remove(idl.lang_absolute_name().name());
                  profile.add(newVer);
                } catch (TIRExceptCreate e) {
                  searchError = "Error in work with profile";
                  return null;
                }
              }
              ID ret = new ID(newVer.name(), newVer.version());
              ret.id = newVer;
              ret.what = kind;
              return ret;
            }
          } catch (TIRExceptLock e) {
            throw new CDLExceptLock("Repository is locked.");
          } catch (RemoteException e) {
            throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
          } catch (CompRepExcept e) {;}
        }
      }
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    
       // search in normal repository
    try {
      Container con = cdlRepository;
      id.sn.names.toFirst();
      for(i=0;i<id.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) id.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else throw new CompRepExcept();
        } else {
          // search last in main branch
          Contained last = con.lookup_lastinbranch((String) id.sn.names.aktual(),"");
          if (last==null) throw new CompRepExcept();
          if (last instanceof Container) con = (Container) last;
          else throw new CompRepExcept();
        }
        id.sn.names.toNext();
      }
      Contained[] sec = con.lookup_name(id.name);
      if (sec!=null && sec.length!=0) {
        if (sec[0].get_def_kind().value() != kind) {
          searchError = "Object with other kind with same name exists in repository.";
          return null;
        }
        searchError = "Object with this name exists in the repository.\nYou must set default version.";
        return null;
      } else {  // here will be scaned enums
        DefinitionKind endk = workRepository.get_spec_def_kind(DefinitionKind.dk_Enum);
        sec = con.contents(endk);
        for (i=0;i<sec.length;i++) {
          EnumMember[] membs = ((EnumDef) sec[i]).members();
          for (j=0;j<membs.length;j++) {
            if (membs[j].name().compareTo(id.name)==0) {
              searchError = "Object with this name exists in the repository.";
              return null;
            }
          }
        }
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}
      // end of search in normal repository
    
    Identification newVer = null;
    try {
      newVer = workRepository.create_initialversion(ID.lang, id.toString());
    } catch (TIRExceptCreate e) {
      searchError = "Can't create initial version";
      return null;
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    switch (kind) {  // add to the profile only objects of this kinds
    case DefinitionKind.dk_Struct:
    case DefinitionKind.dk_Union:
    case DefinitionKind.dk_Enum:
    case DefinitionKind.dk_Interface:
    case DefinitionKind.dk_Frame:
    case DefinitionKind.dk_Exception:
    case DefinitionKind.dk_Typedef:
    case DefinitionKind.dk_Constant:
    case DefinitionKind.dk_Architecture:
      try {
        if (profile!=null)
          profile.add(newVer);
      } catch (TIRExceptCreate e) {
        searchError = "Error in work with profile\n"+e.getMessage();
        return null;
      } catch (RemoteException e) {
        throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
      } catch (TIRExceptLock e) {
//  Output.out.println("****** LL *");    
        throw new CDLExceptLock("Repository is locked.");
      }
    }
    ID ret = null;
    try {
      ret = new ID (newVer.name(), newVer.version());
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    ret.id = newVer;
    ret.what = kind;
    return ret;
  }

  private ID searchForNewTag(FullID id, int kind) throws CDLExceptLock, CDLExceptRemote {
    int i,j;

    try {
       // searching in new objects
    CompContainer con = this;
    id.sn.names.toFirst();
    for(i=0;i<id.sn.names.size();i++) {
      CompContained[] a = con.lookup((String) id.sn.names.aktual());
      if (a==null || a.length==0) 
        throw new CompRepExcept();
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else throw new CompRepExcept();
      } else {
        searchError = "It never would be here.";       
        return null;
      }
      id.sn.names.toNext();
    }
    CompContained sec[] = con.lookup(id.name);
    if (sec!=null && sec.length!=0) {
      searchError = "Object with this name was added in this session.";
      return null;
    } else {   // here will be scaned enums
      sec = con.contents();
      for (i=0;i<sec.length;i++) {   // scan enums 
        if (sec[i].objectKind()==ObjectsKind.o_Enum) {
          if (((CompEnum) sec[i]).members.isIn(id.name)) {
            searchError = "Object with this name and other kind exists";
            return null;
          }
        }
      }
    }
       // end of searching in new objects
    } catch (CompRepExcept e) {;}

    // searching in profiles
    try {
      if (profile!=null) {
        Identification idl = profile.lookup(id.langToString());
        if (idl != null) {
          // search for last version from this def. type
          id.sn.names.toFirst();
          try {
            Container con = cdlRepository;
            for(i=0;i<id.sn.names.size();i++) {
              Contained[] a = con.lookup_name((String) id.sn.names.aktual());
              if (a==null || a.length==0) 
                throw new CompRepExcept();
              if (a.length==1) {
                if (a[0] instanceof Container) con = (Container) a[0];
                else throw new CompRepExcept();
              } else {
                boolean found = false;
                for(j=0;j<a.length;j++) {
                  if (a[j].get_identification().version().compareTo(idl.version())==0) {
                    if ( !(a[j] instanceof Container)) throw new CompRepExcept();
                    found = true;
                    con = (Container) a[j];
                    break;
                  }
                }
                if (!found) {
                  searchError = "Error in profiles.";
                  return null;
                }
              }
              id.sn.names.toNext();
            }
            Contained sec = con.lookup(idl.name(), idl.version());
            if (sec==null) {
              searchError = "Error in profiles.";
              return null;
            }
            if (sec.get_def_kind().value() != kind) {
              searchError = "Object with other kind with same name exists in repository.";
              return null;
            }
            if (con.lookup_tag(id.name, id.tag) != null) {
              searchError = "Object with with same tag exists in repository.";
              return null;
            }
            Contained lst = con.lookup_lastfromversion(idl.name(), idl.version());
            if (lst.get_identification().is_equal(idl)) {  // next version from this
              Identification newVer = null;
              try {
                newVer = workRepository.create_nextversion(idl);
              } catch (TIRExceptCreate e) {
                searchError = "Can't create next version";
                return null;
              }
              switch (kind) {  // add to the profile only objects of this kinds
              case DefinitionKind.dk_Struct:
              case DefinitionKind.dk_Union:
              case DefinitionKind.dk_Enum:
              case DefinitionKind.dk_Interface:
              case DefinitionKind.dk_Frame:
              case DefinitionKind.dk_Exception:
              case DefinitionKind.dk_Typedef:
              case DefinitionKind.dk_Constant:
              case DefinitionKind.dk_Architecture:
                try {
                  profile.remove(idl.lang_absolute_name().name());
                  profile.add(newVer);
                } catch (TIRExceptCreate e) {
                  searchError = "Error in work with profile";
                  return null;
                }
              }
              ID ret = new ID(newVer.name(), newVer.version());
              ret.id = newVer;
              ret.what = kind;
              ret.tag = new String(id.tag);
              ret.isin = IDKind.versiontag;
              return ret;
            } else {  // new branch
              Identification newVer = null;
              try {
                // !!! ok?
                Interactive.writeLine("Input branchtag for "+idl.absolute_name().name()+":");
                String br = Interactive.readLine();
                newVer = workRepository.create_branch(idl, ""); 
              } catch (TIRExceptCreate e) {
                searchError = "Can't create branch version";
                return null;
              } catch (IOException e) {
                searchError = "IO error";
                return null;
              }
              switch (kind) {  // add to the profile only objects of this kinds
              case DefinitionKind.dk_Struct:
              case DefinitionKind.dk_Union:
              case DefinitionKind.dk_Enum:
              case DefinitionKind.dk_Interface:
              case DefinitionKind.dk_Frame:
              case DefinitionKind.dk_Exception:
              case DefinitionKind.dk_Typedef:
              case DefinitionKind.dk_Constant:
              case DefinitionKind.dk_Architecture:
                try {
                  profile.remove(idl.lang_absolute_name().name());
                  profile.add(newVer);
                } catch (TIRExceptCreate e) {
                  searchError = "Error in work with profile";
                  return null;
                }
              }
              ID ret = new ID(newVer.name(), newVer.version());
              ret.id = newVer;
              ret.what = kind;
              ret.tag = new String(id.tag);
              ret.isin = IDKind.versiontag;
              return ret;
            }
          } catch (TIRExceptLock e) {
            throw new CDLExceptLock("Repository is locked.");
          } catch (RemoteException e) {
            throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
          } catch (CompRepExcept e) {;}
        }
      }
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    
       // search in normal repository
    try {
      Container con = cdlRepository;
      id.sn.names.toFirst();
      for(i=0;i<id.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) id.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else throw new CompRepExcept();
        } else {
          // search last in main branch
          Contained last = con.lookup_lastinbranch((String) id.sn.names.aktual(),"");
          if (last==null) throw new CompRepExcept();
          if (last instanceof Container) con = (Container) last;
          else throw new CompRepExcept();
        }
        id.sn.names.toNext();
      }
      Contained[] sec = con.lookup_name(id.name);
      if (sec!=null && sec.length!=0) {
        if (sec[0].get_def_kind().value() != kind) {
          searchError = "Object with other kind with same name exists in repository.";
          return null;
        }
        searchError = "Object with this name exists in the repository.\nYou must set default version.";
        return null;
      } else {  // here will be scaned enums
        DefinitionKind endk = workRepository.get_spec_def_kind(DefinitionKind.dk_Enum);
        sec = con.contents(endk);
        for (i=0;i<sec.length;i++) {
          EnumMember[] membs = ((EnumDef) sec[i]).members();
          for (j=0;j<membs.length;j++) {
            if (membs[j].name().compareTo(id.name)==0) {
              searchError = "Object with this name exists in the repository.";
              return null;
            }
          }
        }
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}
      // end of search in normal repository
    
    Identification newVer = null;
    try {
      newVer = workRepository.create_initialversion(ID.lang, id.toString());
    } catch (TIRExceptCreate e) {
      searchError = "Can't create initial version";
      return null;
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    switch (kind) {  // add to the profile only objects of this kinds
    case DefinitionKind.dk_Struct:
    case DefinitionKind.dk_Union:
    case DefinitionKind.dk_Enum:
    case DefinitionKind.dk_Interface:
    case DefinitionKind.dk_Frame:
    case DefinitionKind.dk_Exception:
    case DefinitionKind.dk_Typedef:
    case DefinitionKind.dk_Constant:
    case DefinitionKind.dk_Architecture:
      try {
        if (profile!=null)
          profile.add(newVer);
      } catch (TIRExceptCreate e) {
        searchError = "Error in work with profile\n"+e.getMessage();
        return null;
      } catch (RemoteException e) {
        throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
      } catch (TIRExceptLock e) {
        throw new CDLExceptLock("Repository is locked.");
      }
    }
    ID ret = null;
    try {
      ret = new ID (newVer.name(), newVer.version());
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    ret.id = newVer;
    ret.what = kind;
    ret.tag = new String(id.tag);
    ret.isin = IDKind.versiontag;
    return ret;
  }

  public ID searchForNewInner(ID id, CompContained aktual) throws CDLExceptLock, CDLExceptRemote {
    int i,j;
    int kind = id.what;
    // set absolute path to "id"
    FullID nid = new FullID(true);
    aktScope.names.toFirst();
    for(i=0;i<aktScope.length();i++) {
      nid.sn.addScope((String)aktScope.names.aktual());
      aktScope.names.toNext();
    }
    nid.name = id.name;
    nid.setVersion(aktual.getIdent().version);

    try {
       // searching in new objects
      CompContainer con = this;
      nid.sn.names.toFirst();
      for(i=0;i<nid.sn.names.size();i++) {
        CompContained[] a = con.lookup((String) nid.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0].isContainer()) con = (CompContainer) a[0];
          else throw new CompRepExcept();
        } else {
          boolean found = false;
          for(j=0;j<a.length;j++) {
            if (a[j].getIdent().version.compareTo(nid.version)==0) {
              if ( ! a[j].isContainer()) throw new CompRepExcept();
              found = true;
              con = (CompContainer) a[j];
              break;
            }
          }
          if (!found) throw new CompRepExcept();
        }
        nid.sn.names.toNext();
      }
      CompContained sec[] = con.lookup(nid.name);
      if (sec!=null && sec.length!=0) {
        searchError = "Object with this name was added in this session.";
        return null;
      } 
    } catch (CompRepExcept e) {
      searchError = "Unexpected error in the objects added in this session";
      return null;
    }
    
    // searching in profiles
    try {
      if (profile!=null) {
        Identification idl = profile.lookup(nid.langToString());
        if (idl != null) {
          // search for last version from this def. type
          nid.sn.names.toFirst();
          try {
            Container con = cdlRepository;
            for(i=0;i<nid.sn.names.size();i++) {
              Contained[] a = con.lookup_name((String) nid.sn.names.aktual());
              if (a==null || a.length==0) 
                throw new CompRepExcept();
              if (a.length==1) {
                if (a[0] instanceof Container) con = (Container) a[0];
                else throw new CompRepExcept();
              } else {
                boolean found = false;
                for(j=0;j<a.length;j++) {
                  if (a[j].get_identification().version().compareTo(idl.version())==0) {
                    if ( !(a[j] instanceof Container)) throw new CompRepExcept();
                    found = true;
                    con = (Container) a[j];
                    break;
                  }
                }
                if (!found) {
                  searchError = "Error in profiles.";
                  return null;
                }
              }
              nid.sn.names.toNext();
            }
            Contained sec = con.lookup(idl.name(), idl.version());
            if (sec==null) {
              searchError = "Error in profiles.";
              return null;
            }
            Identification newVer = null;
            try {
              newVer = workRepository.create_identification(ID.lang, nid.toString(), nid.version);
            } catch (TIRExceptCreate e) {
              searchError = "Can't create version";
              return null;
            } catch (RemoteException e) {
              throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
            }
            switch (kind) {  // remove from profiles
            case DefinitionKind.dk_Struct:
            case DefinitionKind.dk_Union:
            case DefinitionKind.dk_Enum:
            case DefinitionKind.dk_Interface:
            case DefinitionKind.dk_Frame:
            case DefinitionKind.dk_Exception:
            case DefinitionKind.dk_Typedef:
            case DefinitionKind.dk_Constant:
            case DefinitionKind.dk_Architecture:
              try {
                profile.remove(idl.lang_absolute_name().name());
                profile.add(newVer);
              } catch (TIRExceptCreate e) {
                searchError = "Error in work with profile";
                return null;
              }
            }
            ID ret = null;
            try {
              ret = new ID (newVer.name(), newVer.version());
            } catch (RemoteException e) {
              throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
            }
            ret.id = newVer;
            ret.what = kind;
            ret.isin = IDKind.version;
            return ret;
          } catch (TIRExceptLock e) {
            throw new CDLExceptLock("Repository is locked.");
          } catch (RemoteException e) {
            throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
          } catch (CompRepExcept e) {;}
        }
      }
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }

    Identification newVer = null;
    try {
      newVer = workRepository.create_identification(ID.lang, nid.toString(), nid.version);
    } catch (TIRExceptCreate e) {
      searchError = "Can't create version";
      return null;
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    switch (kind) {  // add to the profile only objects of this kinds
    case DefinitionKind.dk_Struct:
    case DefinitionKind.dk_Union:
    case DefinitionKind.dk_Enum:
    case DefinitionKind.dk_Interface:
    case DefinitionKind.dk_Frame:
    case DefinitionKind.dk_Exception:
    case DefinitionKind.dk_Typedef:
    case DefinitionKind.dk_Constant:
    case DefinitionKind.dk_Architecture:
      try {
        if (profile!=null)
          profile.add(newVer);
      } catch (TIRExceptCreate e) {
        searchError = "Error in work with profile\n"+e.getMessage();
        return null;
      } catch (RemoteException e) {
        throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
      } catch (TIRExceptLock e) {
        throw new CDLExceptLock("Repository is locked.");
      }
    }
    ID ret = null;
    try {
      ret = new ID (newVer.name(), newVer.version());
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    ret.id = newVer;
    ret.what = kind;
    ret.isin = IDKind.version;
    return ret;
  }

  // !!! nepouziva se
  private ID searchForNewVersion(FullID id, int kind) throws CDLExceptLock, CDLExceptRemote {
    int i,j;

    try {
       // searching in new objects
    CompContainer con = this;
    id.sn.names.toFirst();
    for(i=0;i<id.sn.names.size();i++) {
      CompContained[] a = con.lookup((String) id.sn.names.aktual());
      if (a==null || a.length==0) 
        throw new CompRepExcept();
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else throw new CompRepExcept();
      } else {
        boolean found = false;
        for(j=0;j<a.length;j++) {
          if (a[j].getIdent().version.compareTo(id.version)==0) {
            if ( ! a[j].isContainer()) throw new CompRepExcept();
            found = true;
            con = (CompContainer) a[j];
            break;
          }
        }
        if (!found) throw new CompRepExcept();
      }
      id.sn.names.toNext();
    }
    CompContained sec[] = con.lookup(id.name);
    if (sec!=null && sec.length!=0) {
      searchError = "Object with this name was added in this session.";
      return null;
    } 
       // end of searching in new objects
    } catch (CompRepExcept e) {;}   
 
       // search in normal repository
    try {
      Container con = cdlRepository;
      id.sn.names.toFirst();
      for(i=0;i<id.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) id.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else throw new CompRepExcept();
        } else {
          searchError = "It never would be here.";
          return null;
        }
        id.sn.names.toNext();
      }
      Contained sec = con.lookup(id.name, id.version);
      if (sec!=null) {
        searchError = "Object with this name exists in the repository.";
        return null;
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}
      // end of search in normal repository

      
    return new ID(id.name, id.version);
  }

  public CompInterface searchFwdIface(ID id) {
    int i;
    // set absolute path to "id"
    FullID nid = new FullID(true);
    aktScope.names.toFirst();
    for(i=0;i<aktScope.length();i++) {
      nid.sn.addScope((String)aktScope.names.aktual());
      aktScope.names.toNext();
    }
    nid.name = id.name;
    nid.setVersion(id.version);

    CompContainer con = this;
    nid.sn.names.toFirst();
    for(i=0;i<nid.sn.names.size();i++) {
      CompContained[] a = con.lookup((String) nid.sn.names.aktual());
      if (a==null || a.length==0) 
        return null;
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else return null;
      } else {
        searchError = "It never would be here.";       
        return null;
      }
      nid.sn.names.toNext();
    }
    CompContained sec[] = con.lookup(nid.name);
    if (sec!=null && sec.length!=0) {  // found object with this name
      if (sec[0].objectKind() == ObjectsKind.o_Interface) {
        if (((CompInterface) sec[0]).forwarddcl) {  // was fwd dcl
          ((CompInterface) sec[0]).forwarddcl = false;
          ((CompInterface) sec[0]).wasforwarddcl = true;
          if (id.isin==IDKind.tag) {
            sec[0].getIdent().tag = new String(id.tag);
            sec[0].getIdent().isin = IDKind.versiontag;
          }
          return (CompInterface) sec[0];
        }
        searchError = "Interface with this name was allready added.";
        return null;
      }
      searchError = "Object with this name was allready added.";
      return null;
    }
    return null;
  }

  /** search for new definition of interface */
/*  public ID searchForIface(ID id) throws CDLExceptLock, CDLExceptRemote {
    int i;
    searchError = null;
    // set absolute path to "id"
    FullID nid = new FullID(true);
    aktScope.names.toFirst();
    for(i=0;i<aktScope.length();i++) {
      nid.sn.addScope((String)aktScope.names.aktual());
      aktScope.names.toNext();
    }
    nid.name = id.name;
    nid.setVersion(id.version);

    //search for fwd declaration 
    try {
      CompContainer con = this;
      nid.sn.names.toFirst();
      for(i=0;i<nid.sn.names.size();i++) {
        CompContained[] a = con.lookup((String) nid.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0].isContainer()) con = (CompContainer) a[0];
          else throw new CompRepExcept();
        } else {
          searchError = "It never would be here.";       
          return null;
        }
        nid.sn.names.toNext();
      }
      CompContained sec[] = con.lookup(nid.name);
      if (sec!=null && sec.length!=0) {  // found object with this name
        searchError = "Object with this name was allready added.";
        return null;
      } 
         // end of searching in new objects
    } catch (CompRepExcept e) {;}
    // it wasn't fwd_dcl
    
    // normal search for new obj
    return searchForNewName(id);
  }*/

  public boolean searchForEnumMember(String name, CompContainer aktual) throws CDLExceptRemote, CDLExceptLock {
    int i;
    // search in aktual container
    CompContained[] aktsec = aktual.contents();
    for (i=0;i<aktsec.length;i++) {
      if (aktsec[i].getIdent().name.compareTo(name)==0)
        return false;
      if (aktsec[i].objectKind()==ObjectsKind.o_Enum) {
        if (((CompEnum) aktsec[i]).members.isIn(name)) {
          return false;
        }
      }
    }
    
    //search in normal repository
    FullID nid = new FullID(true);
    aktScope.names.toFirst();
    for(i=0;i<aktScope.length();i++) {
      nid.sn.addScope((String) aktScope.names.aktual());
      aktScope.names.toNext();
    }
    nid.name = name;
    nid.setVersion(null);
    try {
      Container con = cdlRepository;
      nid.sn.names.toFirst();
      for(i=0;i<nid.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) nid.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else {
            return true;
          }
        } else {
          return true;
        }
        nid.sn.names.toNext();
      }
      Contained[] sec = con.lookup_name(nid.name);
      if (sec!=null && sec.length!=0) {
        return false;
      } else {  // here will be scaned enums
        DefinitionKind endk = workRepository.get_spec_def_kind(DefinitionKind.dk_Enum);
        sec = con.contents(endk);
        for (i=0;i<sec.length;i++) {
          EnumMember[] membs = ((EnumDef) sec[i]).members();
          for (int j=0;j<membs.length;j++) {
            if (membs[j].name().compareTo(nid.name)==0) {
              return false;
            }
          }
        }
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}
      // end of search in normal repository

    return true;
  }

  public CompReffer searchConstant(FullID id) throws CDLExceptRemote, CDLExceptLock {
    // first search in new objects
    int i;
    long numPath = 0;

    if (!id.isAbsolute) {
      // set absolute path to "id"
      FullID nid = new FullID(true);
      aktScope.names.toFirst();
      for(i=0;i<aktScope.length();i++) {
        nid.sn.addScope((String)aktScope.names.aktual());
	aktScope.names.toNext();
      }
      numPath = aktScope.length();
      id.sn.names.toFirst();
      for(i=0;i<id.sn.length();i++) {
        nid.sn.addScope((String)id.sn.names.aktual());
	id.sn.names.toNext();
      }
      nid.name = id.name;
      nid.setVersion(id.version);
      id = nid;
    }
    return searchConstantPriv(id, numPath);
    
  }

  /** called by searchConstant */
  private CompReffer searchConstantPriv(FullID id, long numPath) throws CDLExceptRemote, CDLExceptLock {
    int i,j;
    try {

    CompContainer con = this;
    id.sn.names.toFirst();
    for(i=0;i<id.sn.names.size();i++) {
      CompContained[] a = con.lookup((String) id.sn.names.aktual());
      if (a==null || a.length==0) 
        throw new CompRepExcept();
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else throw new CompRepExcept();
      } else {
        throw new CompRepExcept();
      }
      id.sn.names.toNext();
    }
    CompContained[] sec = con.lookup(id.name);
    if (sec!=null && sec.length!=0) {
      if (sec.length==1) {
        if (sec[0].objectKind() != ObjectsKind.o_Constant) {
          searchError = id + " isn't constant.";
          return null;
        }
        CompReffer ret = new CompReffer();
        ret.what = ObjectsKind.o_Constant;
        ret.ref = new FullID(true);
        ret.ref.name = new String(id.name);
        ret.ref.version = new String(sec[0].getIdent().version);
        ret.ref.isin = IDKind.version;
        ret.isNew = true;
        id.sn.names.toFirst();
        for(j=0;j<id.sn.names.size();j++) {
          ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
          id.sn.names.toNext();
        }
        CompConstant cnst = (CompConstant) sec[0];
        switch (cnst.type.objectKind()) {
        case ObjectsKind.o_Primitive:
          ret.basereffer = ObjectsKind.o_Primitive;
          ret.baserefferprim = ((CompPrimitive) cnst.type).kind;
          break;
        case ObjectsKind.o_none:  // CompReffer (Typedef)
          ret.basereffer = ((CompReffer) cnst.type).basereffer;
          ret.baserefferprim = ((CompReffer) cnst.type).baserefferprim;
          break;
        default:
          ret.basereffer = cnst.type.objectKind();
        }
        return ret;
      } else {
        searchError = id + " isn't constant.";
        return null;
      }
    } 
       // end of searching in new objects
    } catch (CompRepExcept e) {;}
    
    // searching in profiles
    try {
      if (profile != null) {
        Identification idl = profile.lookup(id.langToString());
        if (idl != null) {
          CompReffer ret = new CompReffer();
          ret.ref = new FullID(true);
          ret.ref.name = new String(idl.name());
          ret.ref.version = new String(idl.version());
          ret.ref.isin = IDKind.version;
          ret.isNew = false;
          id.sn.names.toFirst();
          for(j=0;j<id.sn.names.size();j++) {
            ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
            id.sn.names.toNext();
          }
          try {   // searching object with idl in identification for obtaining its kind
            Container con = cdlRepository;
            id.sn.names.toFirst();
            for(i=0;i<id.sn.names.size();i++) {
              Contained[] a = con.lookup_name((String) id.sn.names.aktual());
              if (a==null || a.length==0) 
                throw new CompRepExcept();
              if (a.length==1) {
                if (a[0] instanceof Container) con = (Container) a[0];
                else throw new CompRepExcept();
              } else {
                boolean found = false;
                for(j=0;j<a.length;j++) {
                  if (a[j].get_identification().version().compareTo(idl.version())==0) {
                    if ( !(a[j] instanceof Container)) throw new CompRepExcept();
                    found = true;
                    con = (Container) a[j];
                    break;
                  }
                }
                if (!found) throw new CompRepExcept();
              }
              id.sn.names.toNext();
            }
            Contained sec = con.lookup_lastinbranch(id.name, "");
            if (sec!=null) {
              if (sec.get_def_kind().value() != DefinitionKind.dk_Constant) {
                searchError = id + " isn't constant.";
                return null;
              }
              CompReffer.isConstType(ret, this);  // set basereffer
              ret.what = sec.get_def_kind().value();
            }
          } catch (TIRExceptLock e) {
            throw new CDLExceptLock("Repository is locked.");
          } catch (RemoteException e) {
            throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
          } catch (CompRepExcept e) {
	    searchError = "error in profile";
            return null; 
          }
          return ret;
        }
      }
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } 
    // end of search in profiles

    // search for head version in normal repository
    try {
      Container con = cdlRepository;
      id.sn.names.toFirst();
      for(i=0;i<id.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) id.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else throw new CompRepExcept();
        } else {
          searchError = "More versions of this constant, use profiles.";
          return null;
        }
        id.sn.names.toNext();
      }
      Contained sec = con.lookup_lastinbranch(id.name, "");
      if (sec!=null) {
        if (sec.get_def_kind().value() != DefinitionKind.dk_Constant) {
          searchError = id + " isn't constant.";
          return null;
        }
        CompReffer ret = new CompReffer();
        ret.what = sec.get_def_kind().value();
        ret.ref = new FullID(true);
        ret.ref.name = new String(id.name);
        ret.ref.version = new String(sec.get_identification().version());
        ret.ref.isin = IDKind.version;
        ret.isNew = false;
        id.sn.names.toFirst();
        for(j=0;j<id.sn.names.size();j++) {
          ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
          id.sn.names.toNext();
        }
        CompReffer.isConstType(ret, this);  // set basereffer
        return ret;
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}

    // search in higher name scope (if possible)
    if (numPath > 0) {
      id.sn.names.toFirst();
      for (i=1; i<numPath; i++);
        id.sn.names.toNext();
      id.sn.names.removeAkt();
      return searchConstantPriv(id, numPath-1);
    } else {
      searchError = id.name + ":" + id.version + " doesn't exist";
      return null;
    }
  }

  public CompReffer searchEnumName(FullID id, FullID enumType) throws CDLExceptRemote, CDLExceptLock {
    int i;
    long numPath = 0;
    if (!id.isAbsolute) {
      // set absolute path to "id"
      FullID nid = new FullID(true);
      aktScope.names.toFirst();
      for(i=0;i<aktScope.length();i++) {
        nid.sn.addScope((String)aktScope.names.aktual());
	aktScope.names.toNext();
      }
      numPath = aktScope.length();
      id.sn.names.toFirst();
      for(i=0;i<id.sn.length();i++) {
        nid.sn.addScope((String)id.sn.names.aktual());
	id.sn.names.toNext();
      }
      nid.name = id.name;
      nid.setVersion(id.version);
      id = nid;
    }
    return searchEnumNamePriv(id, enumType, numPath);

  }

  private CompReffer searchEnumNamePriv(FullID id, FullID enumType, long numPath) throws CDLExceptRemote, CDLExceptLock {
    int i, j;
    // first search constant
    // search in new objects
    try {

    CompContainer con = this;
    id.sn.names.toFirst();
    for(i=0;i<id.sn.names.size();i++) {
      CompContained[] a = con.lookup((String) id.sn.names.aktual());
      if (a==null || a.length==0) 
        throw new CompRepExcept();
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else throw new CompRepExcept();
      } else {
        throw new CompRepExcept();
      }
      id.sn.names.toNext();
    }
    CompContained[] sec = con.lookup(id.name);
    if (sec!=null && sec.length!=0) {
      if (sec.length==1) {
        if (sec[0].objectKind() != ObjectsKind.o_Constant) {
          searchError = id + " isn't suitable object.";
          return null;
        }
        CompReffer ret = new CompReffer();
        ret.what = ObjectsKind.o_Constant;
        ret.ref = new FullID(true);
        ret.ref.name = new String(id.name);
        ret.ref.version = new String(sec[0].getIdent().version);
        ret.ref.isin = IDKind.version;
        ret.isNew = true;
        id.sn.names.toFirst();
        for(j=0;j<id.sn.names.size();j++) {
          ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
          id.sn.names.toNext();
        }
        CompConstant cnst = (CompConstant) sec[0];
        switch (cnst.type.objectKind()) {
        case ObjectsKind.o_Primitive:
          ret.basereffer = ObjectsKind.o_Primitive;
          ret.baserefferprim = ((CompPrimitive) cnst.type).kind;
          break;
        case ObjectsKind.o_none:  // CompReffer (Typedef)
          ret.basereffer = ((CompReffer) cnst.type).basereffer;
          ret.baserefferprim = ((CompReffer) cnst.type).baserefferprim;
          break;
        default:
          ret.basereffer = cnst.type.objectKind();
        }
        return ret;
      } else {
        searchError = id + " isn't suitable object.";
        return null;
      }
    } 
       // end of searching in new objects
    } catch (CompRepExcept e) {;}
    
    // searching in profiles
    try {
      if (profile != null) {
        Identification idl = profile.lookup(id.langToString());
        if (idl != null) {
          CompReffer ret = new CompReffer();
          ret.ref = new FullID(true);
          ret.ref.name = new String(idl.name());
          ret.ref.version = new String(idl.version());
          ret.ref.isin = IDKind.version;
          ret.isNew = false;
          id.sn.names.toFirst();
          for(j=0;j<id.sn.names.size();j++) {
            ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
            id.sn.names.toNext();
          }
          try {   // searching object with idl in identification for obtaining its kind
            Container con = cdlRepository;
            id.sn.names.toFirst();
            for(i=0;i<id.sn.names.size();i++) {
              Contained[] a = con.lookup_name((String) id.sn.names.aktual());
              if (a==null || a.length==0) 
                throw new CompRepExcept();
              if (a.length==1) {
                if (a[0] instanceof Container) con = (Container) a[0];
                else throw new CompRepExcept();
              } else {
                boolean found = false;
                for(j=0;j<a.length;j++) {
                  if (a[j].get_identification().version().compareTo(idl.version())==0) {
                    if ( !(a[j] instanceof Container)) throw new CompRepExcept();
                    found = true;
                    con = (Container) a[j];
                    break;
                  }
                }
                if (!found) throw new CompRepExcept();
              }
              id.sn.names.toNext();
            }
            Contained sec = con.lookup_lastinbranch(id.name, "");
            if (sec!=null) {
              if (sec.get_def_kind().value() != DefinitionKind.dk_Constant) {
                searchError = id + " isn't suitable object.";
                return null;
              }
              CompReffer.isConstType(ret, this);  // set basereffer
              ret.what = sec.get_def_kind().value();
            }
          } catch (TIRExceptLock e) {
            throw new CDLExceptLock("Repository is locked.");
          } catch (RemoteException e) {
            throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
          } catch (CompRepExcept e) {
	    searchError = "error in profile";
            return null; 
          }
          return ret;
        }
      }
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } 
    // end of search in profiles

    // search for head version in normal repository
    try {
      Container con = cdlRepository;
      id.sn.names.toFirst();
      for(i=0;i<id.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) id.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else throw new CompRepExcept();
        } else {
          searchError = "More versions, use profiles.";
          return null;
        }
        id.sn.names.toNext();
      }
      Contained sec = con.lookup_lastinbranch(id.name, "");
      if (sec!=null) {
        if (sec.get_def_kind().value() != DefinitionKind.dk_Constant) {
          searchError = id + " isn't suitable object.";
          return null;
        }
        CompReffer ret = new CompReffer();
        ret.what = sec.get_def_kind().value();
        ret.ref = new FullID(true);
        ret.ref.name = new String(id.name);
        ret.ref.version = new String(sec.get_identification().version());
        ret.ref.isin = IDKind.version;
        ret.isNew = false;
        id.sn.names.toFirst();
        for(j=0;j<id.sn.names.size();j++) {
          ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
          id.sn.names.toNext();
        }
        CompReffer.isConstType(ret, this);  // set basereffer
        return ret;
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}

    // search enum member
    // search in new objects
    try {

    CompContainer con = this;
    id.sn.names.toFirst();
    for(i=0;i<id.sn.names.size();i++) {
      CompContained[] a = con.lookup((String) id.sn.names.aktual());
      if (a==null || a.length==0) 
        throw new CompRepExcept();
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else throw new CompRepExcept();
      } else {
        throw new CompRepExcept();
      }
      id.sn.names.toNext();
    }
    CompContained[] sec = con.lookup(enumType.name);
    if (sec!=null && sec.length!=0) {
      if (sec.length==1) {
        if (sec[0].objectKind() != ObjectsKind.o_Enum) {
          searchError = id + " isn't suitable object.";
          return null;
        }
        if (!((CompEnum) sec[0]).members.isIn(id.name)) {
          searchError = id + " isn't member of enum.";
          return null;
        }
        CompReffer ret = new CompReffer();
        ret.what = ObjectsKind.o_Enum;
        ret.ref = new FullID(true);
        ret.ref.name = new String(enumType.name);
        ret.ref.version = new String(sec[0].getIdent().version);
        ret.ref.isin = IDKind.version;
        ret.isNew = true;
        id.sn.names.toFirst();
        for(j=0;j<id.sn.names.size();j++) {
          ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
          id.sn.names.toNext();
        }
        ret.basereffer = ObjectsKind.o_Enum;
        return ret;
      } else {
        searchError = id + " isn't suitable object.";
        return null;
      }
    } 
       // end of searching in new objects
    } catch (CompRepExcept e) {;}

    // searching in profiles
    try {
      if (profile != null) {
        Identification idl = profile.lookup(enumType.langToString());
        if (idl != null) {
          CompReffer ret = new CompReffer();
          ret.ref = new FullID(true);
          ret.ref.name = new String(idl.name());
          ret.ref.version = new String(idl.version());
          ret.ref.isin = IDKind.version;
          ret.isNew = false;
          id.sn.names.toFirst();
          for(j=0;j<id.sn.names.size();j++) {
            ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
            id.sn.names.toNext();
          }
          try {   // searching object with idl in identification for obtaining its kind
            Container con = cdlRepository;
            id.sn.names.toFirst();
            for(i=0;i<id.sn.names.size();i++) {
              Contained[] a = con.lookup_name((String) id.sn.names.aktual());
              if (a==null || a.length==0) 
                throw new CompRepExcept();
              if (a.length==1) {
                if (a[0] instanceof Container) con = (Container) a[0];
                else throw new CompRepExcept();
              } else {
                boolean found = false;
                for(j=0;j<a.length;j++) {
                  if (a[j].get_identification().version().compareTo(idl.version())==0) {
                    if ( !(a[j] instanceof Container)) throw new CompRepExcept();
                    found = true;
                    con = (Container) a[j];
                    break;
                  }
                }
                if (!found) throw new CompRepExcept();
              }
              id.sn.names.toNext();
            }
            Contained sec = con.lookup_lastinbranch(enumType.name, "");
            if (sec!=null) {
              if (sec.get_def_kind().value() != DefinitionKind.dk_Enum) {
                searchError = id + " isn't suitable object.";
                return null;
              }
              boolean found = false;
              EnumMember[] membs = ((EnumDef) sec).members();
              for (i=0;i<membs.length;i++) {
                if (membs[i].name().compareTo(id.name)==0) {
                  found = true;
                  break;
                }
              }
              if (!found) {
                searchError = id + " isn't member of enum.";
                return null;
              }
              ret.what = sec.get_def_kind().value();
            }
          } catch (TIRExceptLock e) {
            throw new CDLExceptLock("Repository is locked.");
          } catch (RemoteException e) {
            throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
          } catch (CompRepExcept e) {
	    searchError = "error in profile";
            return null; 
          }
          return ret;
        }
      }
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } 
    // end of search in profiles

    // search for head version in normal repository
    try {
      Container con = cdlRepository;
      id.sn.names.toFirst();
      for(i=0;i<id.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) id.sn.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else throw new CompRepExcept();
        } else {
          searchError = "More versions, use profiles.";
          return null;
        }
        id.sn.names.toNext();
      }
      Contained sec = con.lookup_lastinbranch(enumType.name, "");
      if (sec!=null) {
        if (sec.get_def_kind().value() != DefinitionKind.dk_Enum) {
          searchError = id + " isn't suitable object.";
          return null;
        }
        boolean found = false;
        EnumMember[] membs = ((EnumDef) sec).members();
        for (i=0;i<membs.length;i++) {
          if (membs[i].name().compareTo(id.name)==0) {
            found = true;
            break;
          }
        }
        if (!found) {
          searchError = id + " isn't member of enum.";
          return null;
        }
        CompReffer ret = new CompReffer();
        ret.what = sec.get_def_kind().value();
        ret.ref = new FullID(true);
        ret.ref.name = new String(enumType.name);
        ret.ref.version = new String(sec.get_identification().version());
        ret.ref.isin = IDKind.version;
        ret.isNew = false;
        id.sn.names.toFirst();
        for(j=0;j<id.sn.names.size();j++) {
          ret.ref.sn.addScope(new String((String)id.sn.names.aktual()));
          id.sn.names.toNext();
        }
        CompReffer.isConstType(ret, this);  // set basereffer
        return ret;
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}

    // search in higher name scope (if possible)
    if (numPath > 0) {
      id.sn.names.toFirst();
      for(i=1;i<numPath;i++)
        id.sn.names.toNext();
      id.sn.names.removeAkt();
      return searchEnumName(id, enumType);
    } else {
      searchError = id.name + ":" + id.version + " doesn't exist";
      return null;
    }
  }

  public CompProvider searchExistNewProvider(String name) {
    Scopename fullname = new Scopename();
    int i;
    aktScope.names.toFirst();
    for (i=0;i<aktScope.length();i++) {
      fullname.addScope((String)aktScope.names.aktual());
      aktScope.names.toNext();
    }
    fullname.addScope(name);

    CompContainer con = this;
    fullname.names.toFirst();
    for(i=0;i<fullname.names.size()-1;i++) {
      CompContained[] a = con.lookup((String) fullname.names.aktual());
      if (a==null || a.length==0) 
        return null;
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else {
          return null;
        }  
      } else {
        return null;
      }
      fullname.names.toNext();
    }
    CompContained sec[] = con.lookup((String) fullname.names.aktual());
    if (sec!=null && sec.length!=0) {
      if (sec[0].objectKind() == ObjectsKind.o_Provider) {
        return (CompProvider) sec[0];
      } else {
        return null;
      }
    }
    return null;
  }

  public ID searchForNewProvider(String name) throws CDLExceptLock, CDLExceptRemote {
    // first search in new objects
    CompContained[] con = this.lookup(name);
    if (con==null || con.length == 0) {
      ;  // continue search in normal rep
    } else {  // something exists
      searchError = "Other objects with this name exist";
      return null;
    }
    
    try {
      Contained[] dcon = cdlRepository.lookup_name(name);
      if (dcon == null || dcon.length==0) { // new Provider
        Identification newVer = null;
        try {
          newVer = workRepository.create_identification(ID.lang, "::"+name,"");
        } catch (TIRExceptCreate e) {
          searchError = "Can't create identification";
          return null;
        } catch (RemoteException e) {
          throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
        }
        ID ret = null;
        try {
          ret = new ID (newVer.name(), newVer.version());
        } catch (RemoteException e) {
          throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
        }
        ret.id = newVer;
        ret.what = ObjectsKind.o_Provider;
        return ret;
      } else {
        if (dcon.length == 1) {
          if (dcon[0].get_def_kind().value() == DefinitionKind.dk_Provider) {
            ID ret = null;
            try {
              Identification idl = dcon[0].get_identification();
              ret = new ID (idl.name(), idl.version());
            } catch (RemoteException e) {
              throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
            }
            ret.exist = true;
            ret.id = null;
            ret.what = ObjectsKind.o_Provider;
            return ret;
          } else {
            searchError = "Other object with this name exists";
            return null;
          }
        } else {
          searchError = "Other objects with this name exist";
          return null;
        }
      }
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
  }

  public CompAModule searchExistNewAMod(String name) {
    Scopename fullname = new Scopename();
    int i;
    aktScope.names.toFirst();
    for (i=0;i<aktScope.length();i++) {
      fullname.addScope((String)aktScope.names.aktual());
      aktScope.names.toNext();
    }
    fullname.addScope(name);

    CompContainer con = this;
    fullname.names.toFirst();
    for(i=0;i<fullname.names.size()-1;i++) {
      CompContained[] a = con.lookup((String) fullname.names.aktual());
      if (a==null || a.length==0) 
        return null;
      if (a.length==1) {
        if (a[0].isContainer()) con = (CompContainer) a[0];
        else {
          return null;
        }  
      } else {
        return null;
      }
      fullname.names.toNext();
    }
    CompContained sec[] = con.lookup((String) fullname.names.aktual());
    if (sec!=null && sec.length!=0) {
      if (sec[0].objectKind() == ObjectsKind.o_AModule) {
        return (CompAModule) sec[0];
      } else {
        return null;
      }
    }
    return null;
  }

  public ID searchForNewAModule(String name) throws CDLExceptLock, CDLExceptRemote {
    Scopename fullname = new Scopename();
    int i;
    aktScope.names.toFirst();
    for (i=0;i<aktScope.length();i++) {
      fullname.addScope((String)aktScope.names.aktual());
      aktScope.names.toNext();
    }
    fullname.addScope(name);

    try {
       // searching in new objects
      CompContainer con = this;
      fullname.names.toFirst();
      for(i=0;i<fullname.names.size()-1;i++) {
        CompContained[] a = con.lookup((String) fullname.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0].isContainer()) con = (CompContainer) a[0];
          else {
            searchError = "This name can not be amodule";       
            return null;
          }  
        } else {
          searchError = "This name can not be amodule";       
          return null;
        }
        fullname.names.toNext();
      }
      CompContained sec[] = con.lookup((String) fullname.names.aktual());
      if (sec!=null && sec.length!=0) {
        searchError = "Object with this name and other kind exists";
        return null;
      } else {   // here will be scaned enums
        sec = con.contents();
        for (i=0;i<sec.length;i++) {   // scan enums 
          if (sec[i].objectKind()==ObjectsKind.o_Enum) {
            if (((CompEnum) sec[i]).members.isIn((String) fullname.names.aktual())) {
              searchError = "Object with this name and other kind exists";
              return null;
            }
          }
        }
      }
       // end of searching in new objects
    } catch (CompRepExcept e) {;}

      // search in normal repository
    try {
      Container con = cdlRepository;
      fullname.names.toFirst();
      for(i=0;i<fullname.names.size()-1;i++) {
        Contained[] a = con.lookup_name((String) fullname.names.aktual());
        if (a==null || a.length==0) 
          throw new CompRepExcept();
        if (a.length==1) {
          if (a[0] instanceof Container) con = (Container) a[0];
          else {
            searchError = "This name can not be amodule";       
            return null;
          }
        } else {
          searchError = "This name can not be amodule";       
          return null;
        }
        fullname.names.toNext();
      }
      Contained[] sec = con.lookup_name((String) fullname.names.aktual());
      if (sec!=null && sec.length!=0) {
        if (sec[0].get_def_kind().value() == DefinitionKind.dk_AModule) {
          ID ret = null;
          try {
            Identification idl = sec[0].get_identification();
            ret = new ID (idl.name(), idl.version());
          } catch (RemoteException e) {
            throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
          }
          ret.exist = true;
          ret.id = null;
          ret.what = ObjectsKind.o_Module;
          return ret;
        } else {
          searchError = "Object with this name exists in the repository.";
          return null;
        }  
      } else {  // here will be scaned enums
        DefinitionKind endk = workRepository.get_spec_def_kind(DefinitionKind.dk_Enum);
        sec = con.contents(endk);
        for (i=0;i<sec.length;i++) {
          EnumMember[] membs = ((EnumDef) sec[i]).members();
          for (int j=0;j<membs.length;j++) {
            if (membs[j].name().compareTo((String) fullname.names.aktual())==0) {
              searchError = "Object with this name exists in the repository.";
              return null;
            }
          }
        }
      }
      
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (CompRepExcept e) {;}
      // end of search in normal repository

    Identification newVer = null;
    try {
      newVer = workRepository.create_identification(ID.lang, fullname.toString(),"");
    } catch (TIRExceptCreate e) {
      searchError = "Can't create identification";
      return null;
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    ID ret = null;
    try {
      ret = new ID (newVer.name(), newVer.version());
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
    ret.id = newVer;
    ret.what = ObjectsKind.o_AModule;
    return ret;
  }

  public boolean abort() {
    boolean ret = false;
    try {
      ret = workRepository.abort();
    } catch (RemoteException e) {
      Output.out.println("Remote exception: "+e.getMessage());
      System.exit(0);
    }
    return ret;
  }

  public boolean commit() {
    boolean ret = false;
    try {
      ret = workRepository.commit();
    } catch (RemoteException e) {
      Output.out.println("Remote exception: "+e.getMessage());
      System.exit(0);
    } catch (TIRExceptCommit e) {
      Output.out.println(e.getMessage());
      e.printStackTrace();
      System.exit(0); 
    }
    return ret;
  }

  /** print actually compiled objects */
  public void printContent() {
    scanContainer(this,0);
  }
  
  /** for printContent() */
  private void scanContainer(CompContainer what, int ind) {
    CompContained con[] = what.contents();
    ID id;
    for (int i=0;i<con.length;i++) {
      intend(ind);
      Output.out.print("+--");
      if (con[i]==null) 
        Output.out.println("! not implemented !");
      else {  
        printKind(con[i].objectKind());
        id = con[i].getIdent();
        Output.out.println(" \""+id.name+"\" "+id.version);
	if ((con[i]) instanceof CompInterface) {
          intend(ind);
	  Output.out.println(((CompInterface) con[i]).protocolToText());
	}
	if ((con[i]) instanceof CompFrame) {
          intend(ind);
	  Output.out.println(((CompFrame) con[i]).protocolToText());
	}
        if ((con[i]) instanceof CompContainer)
          scanContainer((CompContainer) con[i], ind+1);
      }
    }
  }

  /** for printContent() */
  private static void intend(int in) {
    for(int i=0;i<in;i++)
      Output.out.print("|  ");
  }

  /** for printContent() */
  private static void printKind(int ok) {
    switch (ok) {
      case ObjectsKind.o_Attribute : Output.out.print("Attribute"); break;
      case ObjectsKind.o_Constant : Output.out.print("Constant"); break;
      case ObjectsKind.o_Exception : Output.out.print("Exception"); break;
      case ObjectsKind.o_Interface : Output.out.print("Interface"); break;
      case ObjectsKind.o_Module : Output.out.print("Module"); break;
      case ObjectsKind.o_Operation : Output.out.print("Operation"); break;
      case ObjectsKind.o_Typedef : Output.out.print("Typedef"); break;
      case ObjectsKind.o_Struct : Output.out.print("Struct"); break;
      case ObjectsKind.o_Union : Output.out.print("Union"); break;
      case ObjectsKind.o_Enum : Output.out.print("Enum"); break;
      case ObjectsKind.o_Primitive : Output.out.print("Primitive"); break;
      case ObjectsKind.o_String : Output.out.print("String"); break;
      case ObjectsKind.o_Sequence : Output.out.print("Sequence"); break;
      case ObjectsKind.o_Array : Output.out.print("Array"); break;
      case ObjectsKind.o_Repository : Output.out.print("Repository"); break;
      case ObjectsKind.o_Frame : Output.out.print("Frame"); break;
      case ObjectsKind.o_Architecture : Output.out.print("Architecture"); break;
      case ObjectsKind.o_Provides : Output.out.print("Provides"); break;
      case ObjectsKind.o_Requires : Output.out.print("Requires"); break;
      case ObjectsKind.o_Property : Output.out.print("Property"); break;
      case ObjectsKind.o_Implements : Output.out.print("Implements"); break;
      case ObjectsKind.o_Inst : Output.out.print("Inst"); break;
      case ObjectsKind.o_Bind : Output.out.print("Bind"); break;
      case ObjectsKind.o_Protocol : Output.out.print("Protocol"); break;
      case ObjectsKind.o_StructMember : Output.out.print("StructMember"); break;
      case ObjectsKind.o_EnumMember : Output.out.print("EnumMember"); break;
      case ObjectsKind.o_UnionMember : Output.out.print("UnionMember"); break;
      case ObjectsKind.o_Expression : Output.out.print("Expression"); break;
      case ObjectsKind.o_ProtocolOper : Output.out.print("ProtocolOper"); break;
      case ObjectsKind.o_ValueGen : Output.out.print("ValueGen"); break;
      case ObjectsKind.o_ValueGenElem : Output.out.print("ValueGenOper"); break;
      case ObjectsKind.o_BindOper : Output.out.print("BindOper"); break;
      case ObjectsKind.o_Provider : Output.out.print("Provider"); break;
      case ObjectsKind.o_AModule : Output.out.print("AModule"); break;
      case ObjectsKind.o_Wstring : Output.out.print("Wstring"); break;
      case ObjectsKind.o_Fixed : Output.out.print("Fixed"); break;
    }
  }

  public void toNormal() throws CDLExceptRemote, CDLExceptToNormal, CDLExceptLock {
    java.util.ArrayList delayed = new java.util.ArrayList();
    addToNormal(cdlRepository, cdlRepository, delayed);
    for (int i=0; i<delayed.size();i+=2) {
      CompInterface ci = (CompInterface) delayed.get(i);
      Container obj = (Container) delayed.get(i+1);
      ci.addToNormalDelayed(obj, cdlRepository, delayed);
    }
  }

  public void addToNormal(Container obj, CDLRepository normRep, java.util.ArrayList delayed) throws CDLExceptToNormal, CDLExceptRemote, CDLExceptLock {
    cont.toFirst();
    try {
      for (int i=0;i<cont.size(); i++) {
        CompContained akt = (CompContained) cont.aktual();
        switch (akt.objectKind()) {
        // *****************
        case ObjectsKind.o_Module:
          Contained[] cmod = obj.lookup_name(akt.getIdent().name);
          if ((cmod == null) || (cmod.length == 0)) { //create ne module
            ModuleDef mod = null;
            try {
              mod = ((CDLContainer) obj).create_module(akt.getIdent().id);
            } catch (TIRExceptCreate ecr) {
              throw new CDLExceptToNormal("Can't create module "+((CompContained)akt).fullName());
            }
            if (mod==null)
              throw new CDLExceptToNormal("Can't create module "+((CompContained)akt).fullName());
            ((CompContainer) akt).addToNormal(mod, normRep, delayed);
          } else { // use old module
            if (cmod.length!=1)
              throw new CDLExceptToNormal("Unexpected error");
            ((CompContainer) akt).addToNormal((Container) cmod[0], normRep, delayed);
          }
          break;
        // *****************
        case ObjectsKind.o_Typedef:
          try {
            TypedefDef td = ((CDLContainer) obj).create_typedef(akt.getIdent().id, ((CompTypedef) akt).type.toNormal(normRep));
            if (td == null) 
              throw new CDLExceptToNormal("Can't create typedef "+((CompContained)akt).fullName());
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create typedef "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Enum:
          try {
            EnumDef en = ((CDLContainer) obj).create_enum(akt.getIdent().id);
            if (en == null) 
              throw new CDLExceptToNormal("Can't create enum "+((CompContained)akt).fullName());
            ((CompEnum) akt).addToNormal(en);
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create enum "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Struct:
          try {
            StructDef str = ((CDLContainer) obj).create_struct(akt.getIdent().id);
            if (str == null) 
              throw new CDLExceptToNormal("Can't create struct "+((CompContained)akt).fullName());
            ((CompStruct) akt).addToNormal(str, normRep, delayed);
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create struct "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Union:
          try {
            UnionDef un = ((CDLContainer) obj).create_union(akt.getIdent().id, ((CompUnion) akt).switch_type.toNormal(normRep));
            if (un == null) 
              throw new CDLExceptToNormal("Can't create union "+((CompContained)akt).fullName());
            ((CompUnion) akt).addToNormal(un, normRep, delayed);
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create union "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Exception:
          try {
            ExceptionDef str = ((CDLContainer) obj).create_exception(akt.getIdent().id);
            if (str == null) 
              throw new CDLExceptToNormal("Can't create exception "+((CompContained)akt).fullName());
            ((CompException) akt).addToNormal(str, normRep, delayed);
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create exception "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Constant:
          try {
            ConstantDef cnst = ((CDLContainer) obj).create_constant(akt.getIdent().id, ((CompConstant) akt).type.toNormal(normRep), ((CompExprOper) ((CompConstant)akt).expr).toNormal(normRep));
            if (cnst == null) 
              throw new CDLExceptToNormal("Can't create constant "+((CompContained)akt).fullName());
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create constant "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Interface:
          try {
            /*InterfaceDef[] base = null;
            if (((CompInterface) akt).baseIfaces != null) {
              ((CompInterface) akt).baseIfaces.toFirst();
              base = new InterfaceDef [((CompInterface) akt).baseIfaces.size()];
              for (int j=0;j<((CompInterface) akt).baseIfaces.size();j++) {
                base[j] = (InterfaceDef) ((CompType) ((CompInterface) akt).baseIfaces.aktual()).toNormal(normRep);
                ((CompInterface) akt).baseIfaces.toNext();
              }
            } else {
              base =  new InterfaceDef [0];
            } */
            InterfaceDef iface = ((CDLContainer) obj).create_interface(akt.getIdent().id /*,base*/);
            if (iface == null) 
              throw new CDLExceptToNormal("Can't create interface "+((CompContained)akt).fullName());
            ((CompInterface) akt).addToNormal(iface, normRep, delayed);
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create interface "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Frame:
          try {
            FrameDef frm = ((CDLContainer) obj).create_frame(akt.getIdent().id, ((CompFrame)akt).isSystem);
            if (frm == null) 
              throw new CDLExceptToNormal("Can't create frame "+((CompContained)akt).fullName());
            ((CompFrame) akt).addToNormal(frm, normRep, delayed);
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create frame "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Provider:
          Contained[] cprov = obj.lookup_name(akt.getIdent().name);
          if ((cprov == null) || (cprov.length == 0)) { //create ne module
            ProviderDef prov = null;
            try {
              prov = ((CDLRepository) obj).create_provider(akt.getIdent().id);
            } catch (TIRExceptCreate ecr) {
              throw new CDLExceptToNormal("Can't create provider "+((CompContained)akt).fullName());
            }
            if (prov==null)
              throw new CDLExceptToNormal("Can't create provider "+((CompContained)akt).fullName());
            ((CompContainer) akt).addToNormal(prov, normRep, delayed);
          } else { // use old module
            if (cprov.length!=1)
              throw new CDLExceptToNormal("Unexpected error");
            ((CompContainer) akt).addToNormal((Container) cprov[0], normRep, delayed);
          }
          break;
        default:
          throw new CDLExceptToNormal("Unexcepted kind of object");
        }
        cont.toNext();
      }
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    }
  }
}

// for searching refference to the type or new name 
/*
class DefaultTypes {
  Scopename path;
  String name;
  String version;
  int type;

  public DefaultTypes(String fullname, String version) {
    type = 0;
    version = new String(version);
    int i;
    int count = 0;
   
    for (i=0;i<name.length();i++) {
      if (name.charAt(i)==':' && name.charAt(i+1)==':') 
        count++;
    }
    int j = 0;
    for(i=0;i<count;i++) {
      StringBuffer str = new StringBuffer();
      while (name.charAt(j)==':')
        j++;
      while (name.charAt(j)!=':' && j<name.length()) {
        str = str.append(name.charAt(j));
        j++;
      }
      if (i==count-1)
        this.name = str.toString();
      else 
        path.addScope(str.toString());
    }   
  }

  public String toString() {
    if (path.length()==0)
      return "::"+name+" "+version;
    else  
      return path.toString()+"::"+name+" "+version;
  }

  // test, if this name and version exist
  public boolean test(CDLRepository rep) throws CDLExceptLock, CDLExceptRemote {
    try {
      int i,j;
      Container con = rep;
      path.names.toFirst();
      for(i=0;i<path.names.size();i++) {
        Contained[] a = con.lookup_name((String) path.names.aktual());
        if (a==null || a.length==0) 
          return false;
        if (a.length==1) {
          if (CompRepository.isDefContainer(a[0].get_def_kind().value())) con = (Container) a[0];
          else return false;
        } else {
          boolean found = false;
          for(j=0;j<a.length;j++) {
            if (a[j].get_identification().version().compareTo(version)==0) {
              if ( ! CompRepository.isDefContainer(a[j].get_def_kind().value())) return false;
              found = true;
              con = (Container) a[j];
              break;
            }
          }
          if (!found) return false;
        }
        path.names.toNext();
      }
      Contained sec = con.lookup(name, version);
      if (sec!=null) {
        type = sec.get_def_kind().value();
        return true;
      } else {
        return false;
      }
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
  }
} */
