/* $Id: Expressions.java,v 2.0 2002/02/15 13:55:00 hnetynka Exp $ */
package SOFA.SOFAnode.Made.CodeGen;

import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.RemoteException;
 
/** Some functions for handling expressions.
  * 
  * @author Petr Hnetynka
  */
class Expressions {
  /** Add all referenced objects (constants, enums) that are in <tt>expr</tt> to the <tt>list</tt>.  
    *
    * @param     expr  scanned expression
    * @param     list  add all objects to this list
    * @exception RemoteException  remote exception 
    * @exception TIRAccessException  error in access to the TIR
    */
  static void scanExpr(ExprOperDef expr, ObjectList list) throws RemoteException, TIRAccessException {
    switch (expr.get_eok_kind().value()) {
    case ExprOperKind.eok_or:
    case ExprOperKind.eok_xor:
    case ExprOperKind.eok_and:
    case ExprOperKind.eok_shr:
    case ExprOperKind.eok_shl:
    case ExprOperKind.eok_plus:
    case ExprOperKind.eok_minus:
    case ExprOperKind.eok_mul:
    case ExprOperKind.eok_div:
    case ExprOperKind.eok_mod:
      ExprBinOperationDef bop = (ExprBinOperationDef) expr;
      scanExpr(bop.operand1(), list);
      scanExpr(bop.operand2(), list);
      break;
    case ExprOperKind.eok_unminus:
    case ExprOperKind.eok_unplus:
    case ExprOperKind.eok_untilde:
      scanExpr(((ExprUnOperationDef) expr).operand(), list);
      break;
    case ExprOperKind.eok_short:
    case ExprOperKind.eok_long:
    case ExprOperKind.eok_longlong:
    case ExprOperKind.eok_unsigshort:
    case ExprOperKind.eok_unsiglong:
    case ExprOperKind.eok_unsiglonglong:
    case ExprOperKind.eok_float:
    case ExprOperKind.eok_double:
    case ExprOperKind.eok_octet:
    case ExprOperKind.eok_char:
    case ExprOperKind.eok_wchar:
    case ExprOperKind.eok_string:
    case ExprOperKind.eok_wstring:
    case ExprOperKind.eok_default:
    case ExprOperKind.eok_boolean:
    case ExprOperKind.eok_longdouble:
    case ExprOperKind.eok_fixed:
      break;
    
    case ExprOperKind.eok_property:
      break;
    case ExprOperKind.eok_constant:
      list.addObject(((ExprConstant) expr).ref_const());
      break;
    case ExprOperKind.eok_enum:
      list.addObject(((ExprEnum) expr).enum());
      break;

    }
  }
}
