/* $Id: ID.java,v 2.0 2002/02/15 13:55:01 hnetynka Exp $ */
package SOFA.SOFAnode.Made.CodeGen;
 
import java.util.ArrayList;

/** Representing ID of architecture given on command line.
  * 
  * @author Petr Hnetynka
  */
public class ID {
  ArrayList name;
  String ver;

  /** Parse textual representation of id.  
    *
    * @param nm   textual representation of id in format <tt>absolute_name?version</tt> or <tt>absolute_name</tt>
    * @exception BadIDException  if <tt>nm</tt> is in bad format
    */
  public ID(String nm) throws BadIDException {
    name = new ArrayList();
    int iOfQ = nm.indexOf('?');
    if (iOfQ == -1) { // absolute name without version
      parseAN(nm, name);
    } else {  // absolute name with version
      String absnm = nm.substring(0, iOfQ);
      ver = nm.substring(iOfQ+1);
      if (ver.length()==0 || ver.indexOf('?')!=-1)
        throw new BadIDException();
      parseAN(absnm, name);
    }
  }

  /** Parse absolute name in nm.
    *
    * @exception BadIDException   if <tt>nm</tt> is in bad format
    * @param nm   string with absolute name
    * @param name array for name elements (must be allocated)
    */
  private void parseAN(String nm, ArrayList name) throws BadIDException {
    StringBuffer sn = new StringBuffer();
    boolean dv = false;
    for (int i=0;i<nm.length();i++) {
      if (dv && nm.charAt(i)!=':') 
        throw new BadIDException();
      if (dv) { // nm.charAt(i) == ':'
        if (name.size()!=0 || sn.length()!=0) { // it's not double colon in the begining of name
          if (sn.length()==0)  // more than two colons
            throw new BadIDException();
          name.add(sn.toString());
          sn.delete(0, sn.length());
        }
        dv = false;
        continue;
      }
      if (nm.charAt(i)==':') {
        dv = true;
        continue;
      }
      sn.append(nm.charAt(i));
    }
    if (sn.length() != 0)  // append last item
      name.add(sn.toString());
    else 
      throw new BadIDException();
  }

  /** Returns textual representation of id.
    */
  public String toString() {
    StringBuffer ret = new StringBuffer("::");
    for (int i=0; i<name.size();i++) {
      ret.append((String) name.get(i));
      if (i+1 != name.size())
        ret.append("::");
    }
    if (ver != null && ver.length() != 0) {
      ret.append("?");
      ret.append(ver);
    }
    return ret.toString();
  }

  /** Returns absolute name (without version)
    */
  public String absoluteName() {
    StringBuffer ret = new StringBuffer("::");
    for (int i=0; i<name.size();i++) {
      ret.append((String) name.get(i));
      if (i+1 != name.size())
        ret.append("::");
    }
    return ret.toString();
  }

  /** Returns number of elements in the absolute name of id.
    */
  public int size() {
    return name.size();
  }

  /** Returns version of id.
    */
  public String version() {
    return ver;
  }

  /** Returns i-th element of absolute name
    *
    * @param index  index of element to return
    * @exception IndexOutOfBoundsException if index is out of range
    */
  public String get(int index) {
    return (String) name.get(index);
  }
}
