/* $Id: ArchitectureDefImpl.java,v 2.0 2002/02/15 13:55:26 hnetynka Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.*;

public class ArchitectureDefImpl extends ContainerImpl implements ArchitectureDef, TIRImplObject, SContained {
  protected Identification id;
  protected Container parent;
  protected Repository rep;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  FrameDef frm;
  CDLType loadFrm;
  boolean system;

  ArchitectureDefImpl normal;
  long workId;
  
  public ArchitectureDefImpl(Identification ident, Container in, Repository inrep, FrameDef frame, int state, boolean isSystem, long workId) throws RemoteException {
    id = ident;
    parent = in;
    rep = inrep;
    frm = frame;
    system = isSystem;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Architecture);
    stKindImpl = new StateKindImpl(state);
    normal = null;
    if (state == StateKind.sk_work)
      this.workId = workId;
    else 
      this.workId = -1L;
  }

  public ArchitectureDefImpl(Container in, Repository inrep) throws RemoteException {
    id = null;
    parent = in;
    rep = inrep;
    frm = null;
    system = false;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Architecture);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    normal = null;
    this.workId = -1L;
  }

  public ArchitectureDefImpl(ArchitectureDefImpl a, Container in, Repository inrep, long workId) throws RemoteException {
    parent = in;
    rep = inrep;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Architecture);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    normal = a;
    frm = normal.frm;
    system = normal.system;
    id = a.id;
    this.workId = workId;
  }

  /* form interface Contained */
  public Identification get_identification() throws RemoteException {
    return id;
  }

  /* form interface Contained */
  public Container get_defined_in() throws RemoteException {
    return parent;
  }

  /* form interface Contained */
  public Container get_containing_repository() throws RemoteException {
    return rep;
  }

  /* form interface Contained */
  public AbsoluteName get_absolute_name() throws RemoteException {
    AbsoluteNameImpl absName;
    if (rep == parent) {  // object is contained in repository
      absName = new AbsoluteNameImpl("::"+id.name());
    } else {
      absName = new AbsoluteNameImpl(((SContained)parent).get_absolute_name().name()+"::"+id.name());
    }
    return (AbsoluteName) absName;
  }

  public FrameDef frame() throws RemoteException {
    return frm;
  }

  public boolean is_system() throws RemoteException {
    return system;
  }

  public void tag(String t) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (stKindImpl.value()!=StateKind.sk_work)
      throw new TIRExceptCreate("you can call create method on work object");
    String ebt = id.branchTag();
    if (t.compareTo(ebt)==0)
      throw new TIRExceptCreate("This tag exists");
    String[] et = id.tag();
    int i,j;
    for (i=0;i<et.length;i++) {
      if (t.compareTo(et[i])==0)
       throw new TIRExceptCreate("This tag exists");       
    }
    Contained[] sibl = parent.lookup_name(id);
    for (j=0;j<sibl.length;j++) {
      Identification idl = sibl[j].get_identification();
      
      ebt = idl.branchTag();
      if (t.compareTo(ebt)==0)
        throw new TIRExceptCreate("This tag exists");
      et = idl.tag();
      for (i=0;i<et.length;i++) {
        if (t.compareTo(et[i])==0)
          throw new TIRExceptCreate("This tag exists");       
      } 
    }
    ((SIdentification) id).tag(t);
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  private void fromNormObj() throws RemoteException, TIRExceptLock {
    if (normal==null)
      return;
    if (((WorkRepositoryImpl)rep).lock.isLocked())
      throw new TIRExceptLock("Repository is locked");
    int i;
    LiItem akt;
    akt = normal.firstChild;
    for(i=0;i<normal.numOfItems;i++) {
      addListItem(WorkRepositoryImpl.newWorkFromNormal(akt.obj,rep,this,workId));
      akt = akt.next;
    }
    hasCont = true;
  }

  public Contained[] contents(DefinitionKind type) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.contents(type);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.contents(type);     
    }
  }

  public Contained lookup(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(id);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(id);
    }
  }

  public Contained lookup(String name, String ver) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(name, ver);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(name, ver);
    }
  }

  public Contained[] lookup_name(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(id);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(id);
    }
  }

  public Contained[] lookup_name(String id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(id);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else {
        return super.lookup_name(id);
      }
    }
  }

  public Contained lookup_tag(String name, String tag) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_tag(name, tag);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_tag(name, tag);
    }
  }

  public Contained[] lookup_branchtag(String name, String brtag) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_branchtag(name, brtag);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_branchtag(name, brtag);
    }
  }

  public Contained lookup_lastinbranch(String name, String brtag) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastinbranch(name, brtag);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastinbranch(name, brtag);
    }
  }
  
  public Contained lookup_lastfromversion(String name, String version) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastfromversion(name, version);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastfromversion(name, version);
    }
  }

  public Contained[] scontents(DefinitionKind type) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      return super.scontents(type);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.scontents(type);
    }
  }

  public Contained slookup(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
        return super.slookup(id);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup(id);
    }
  }

  public Contained slookup(String name, String version) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
        return super.slookup(name, version);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup(name, version);
    }
  }

  public Contained[] slookup_name(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
        return super.slookup_name(id);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup_name(id);
    }
  }

  public Contained[] slookup_name(String name) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      return super.slookup_name(name);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup_name(name);
    }
  }

  public InstDef create_inst(String name, CDLType type, int mode) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    if (type==null)
      return null;
    else {
      if (type.get_state().value()==StateKind.sk_work ) { 
        IdentificationImpl id = new IdentificationImpl(((SIdentification)this.id).language(), this.id.absolute_name().name()+"::"+name,this.id.version());
        Contained[] a = lookup_name(id.name());
        if (a.length != 0) {
          if (a[0].get_def_kind().value() != DefinitionKind.dk_Inst)
	    throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	  boolean found = false;  
	  for (int i=0;i<a.length;i++) {
	    if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	      found = true;
	      break;
	    }
	  }
	  if (found) 
	    throw new TIRExceptCreate("Object with same identification exists in repository.");
        }
        // creating new object
        InstDef ret = new InstDefImpl(id, this, rep, type, new InstModeImpl(mode), workId);
        addListItem(ret); // adding object to list
        return ret;       // return object
      } else {
        throw new TIRExceptCreate("Given object isn't work.");
      }
    }
  }
  
  public BindDef create_bind(BindOperDef lhs, BindOperDef rhs, int mode) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    if (rhs==null || lhs==null)
      return null;
    else {
      if (rhs.get_state().value()==StateKind.sk_work &&
          lhs.get_state().value()==StateKind.sk_work) { 
        IdentificationImpl id = new IdentificationImpl(((SIdentification)this.id).language(), this.id.absolute_name().name()+"::Bind"+numOfItems,this.id.version());
        Contained[] a = lookup_name(id.name());
        if (a.length != 0) {
          if (a[0].get_def_kind().value() != DefinitionKind.dk_Bind)
	    throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	  boolean found = false;  
	  for (int i=0;i<a.length;i++) {
	    if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	      found = true;
	      break;
	    }
	  }
	  if (found) 
	    throw new TIRExceptCreate("Object with same identification exists in repository.");
        }
        // creating new object
        BindDef ret = new BindDefImpl(id, this, rep, lhs,rhs,new BindModeImpl(mode), "CSProcCall");
        addListItem(ret); // adding object to list
        return ret;       // return object
      } else {
        throw new TIRExceptCreate("Given object isn't work.");
      }
    }
  }

  public BindDef create_bind(BindOperDef lhs, BindOperDef rhs, int mode, String using) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    if (rhs==null || lhs==null || using == null)
      return null;
    else {
      if (using.compareTo("CSProcCall")!=0 && using.compareTo("DataStream")!=0 && using.compareTo("EventPassing")!=0)
        throw new TIRExceptCreate("Bad type of connector.");
      if (rhs.get_state().value()==StateKind.sk_work &&
          lhs.get_state().value()==StateKind.sk_work) { 
        IdentificationImpl id = new IdentificationImpl(((SIdentification)this.id).language(), this.id.absolute_name().name()+"::Bind"+numOfItems,this.id.version());
        Contained[] a = lookup_name(id.name());
        if (a.length != 0) {
          if (a[0].get_def_kind().value() != DefinitionKind.dk_Bind)
	    throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	  boolean found = false;  
	  for (int i=0;i<a.length;i++) {
	    if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	      found = true;
	      break;
	    }
	  }
	  if (found) 
	    throw new TIRExceptCreate("Object with same identification exists in repository.");
        }
        // creating new object
        BindDef ret = new BindDefImpl(id, this, rep, lhs,rhs,new BindModeImpl(mode), new String(using));
        addListItem(ret); // adding object to list
        return ret;       // return object
      } else {
        throw new TIRExceptCreate("Given object isn't work.");
      }
    }
  }

  public PropertyDef create_property(String name, CDLType type) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    if (name==null || type==null)
      return null;
    int i;  
    if (type.get_state().value()!=StateKind.sk_work )  
      throw new TIRExceptCreate("Given objects aren't work.");
    // set version of substructure same as this version  
    Identification id = new IdentificationImpl(((SIdentification)this.id).language(), this.id.absolute_name().name()+"::"+name,this.id.version());
    Contained[] a = lookup_name(id.name());
    if (a.length != 0) {
      throw new TIRExceptCreate("Property with this name exists.");
    }
    // creating new object
    PropertyDef ret = new PropertyDefImpl(id, this, rep, type, workId);
    addListItem(ret); // adding object to list
    return ret;       // return object
  }

  public BindOperProvReq create_bindoperprovreq() throws RemoteException, TIRExceptCreate {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    return new BindOperProvReqImpl(StateKind.sk_work);
  }

  public BindOperSub create_bindopersub() throws RemoteException, TIRExceptCreate {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    return new BindOperSubImpl(StateKind.sk_work);
  }

  /*from interface TIRImplObject */
  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      st.curOutFile = new DataOutputStream(new FileOutputStream(st.currentFile));
      ((IdentificationImpl)id).save(st.curOutFile);   
      // saving frame
      if (frm == null) {
        st.curOutFile.writeBoolean(false);
      } else {
        st.curOutFile.writeBoolean(true);
        Identification idl = ((Contained)frm).get_identification();
        Storage.writeString(st.curOutFile, idl.lang_absolute_name().name());
        Storage.writeString(st.curOutFile, idl.name());
        Storage.writeString(st.curOutFile, idl.version());
      }
      // end of saving frame
      st.curOutFile.writeBoolean(system);
      st.curOutFile.writeInt(numOfItems);
      LiItem akt = firstChild;
      for(int i=0;i<numOfItems;i++) {  // saving all childs
        st.curOutFile.writeInt(akt.obj.get_def_kind().value());
        ((TIRImplObject)(akt.obj)).save(st);
	akt = akt.next;
      }
      st.curOutFile.close();
    } catch (IOException e) {
      throw new TIRExceptStorage("Access error in "+st.current+".");
    }
  }

  /*from interface TIRImplObject */
  public void load(Storage st) throws RemoteException, TIRExceptStorage{
    try {
      st.curInFile = new DataInputStream(new FileInputStream(st.currentFile));
      id = new IdentificationImpl();
      ((IdentificationImpl)id).load(st.curInFile);
      //loading frame
      if (st.curInFile.readBoolean()) {
        String absn = Storage.readString(st.curInFile);
        String name = Storage.readString(st.curInFile);
        String ver = Storage.readString(st.curInFile);
        loadFrm = new NotYetInRep(absn,name,ver);
      } else {
        loadFrm = null;
      }
      // end of loading frame
      system = st.curInFile.readBoolean();
      int num = st.curInFile.readInt();
      int dkind;
      TIRImplObject obj;
      for (int i=0;i<num;i++) {
        dkind = st.curInFile.readInt();
	switch (dkind) {
	  case DefinitionKind.dk_Inst:
	    obj = new InstDefImpl(this, rep);
	    obj.load(st);
	    addListItem((TIRObject)obj);
	    break;
          case DefinitionKind.dk_Bind:
	    obj = new BindDefImpl(this, rep);
	    obj.load(st);
	    addListItem((TIRObject)obj);
	    break;
          case DefinitionKind.dk_Property:
	    obj = new PropertyDefImpl(this, rep);
	    obj.load(st);
	    addListItem((TIRObject)obj);
            break;
	  default:
            throw new TIRExceptStorage("Unexpected kind of object in \""
	                              +st.currentFile+"\".");
	} // -- end of switch
      }
      st.curInFile.close();
    } catch (IOException e) {
      throw new TIRExceptStorage("Access error in "+st.current+".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    try {
    
      if ((loadFrm!=null) && (loadFrm.get_def_kind().value() == DefinitionKind.dk_none)) {   // is tp NotYetInRep?
        AbsoluteName nm = ((NotYetInRep) loadFrm).nm;
        Container con = r;// begin search from repository
        Contained[] rt;
        Contained ret;
        IdentificationImpl id;
        for (int i=0;i<nm.size()-1;i++) {  // last element is name of frame
  	id = new IdentificationImpl(nm.elementAt(0),"::"+nm.elementAt(i),"");
          rt = ((SContainer)con).slookup_name(id);
  	if (rt.length==0) throw new TIRExceptStorage("Can't find frame " + nm.name());
        if (rt.length==1) {
          con = (Container) rt[0];
  	} else {
  	  throw new TIRExceptStorage("Can't find frame " + nm.name());
  	}
        }
        if ((ret = ((SContainer)con).slookup(((NotYetInRep) loadFrm).id))==null)
        throw new TIRExceptStorage("Can't find frame " + nm.name());
        frm = (FrameDef) ret;
        loadFrm = null;
      }

    } catch (TIRExceptLock e) {;}
  
    LiItem akt = firstChild;
    for(int i=0;i<numOfItems;i++) {
      ((TIRImplObject) akt.obj).postLoad(r);
      akt = akt.next;
    }
  }

  public boolean isNew() {
    return ((stKindImpl.value()==StateKind.sk_work) && normal==null);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    LiItem akt;
    int i;
    akt = firstChild;
    for (i=0;i<numOfItems;i++) {
      ((TIRImplObject)akt.obj).canCommit();
      akt = akt.next;
    }
  }

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    if (isNew()) {  
      stKindImpl.toNormal();
      if (frm!=null) {
        Identification idl = frm.get_identification();
        loadFrm = new NotYetInRep(idl.lang_absolute_name().name(), idl.name(), idl.version());
        frm = null;
      } else {
        loadFrm = null;
      }
      LiItem akt = firstChild;
      for (int i=0;i<numOfItems;i++) {
        ((TIRImplObject)akt.obj).doCommit(this,rep);
        akt = akt.next;     
      }
      parent = in;
      this.rep = rep;
    } else {
      normal = null;
    }
  }

  public void doAbort(long workId) throws RemoteException {
    LiItem akt = firstChild;
    for(int i=0;i<numOfItems;i++) {
      if (((TIRImplObject)akt.obj).isNew()) 
        ((TIRImplObject)akt.obj).doAbort(workId);
      akt = akt.next;
    }
  }
}
