/* $Id: ExprEnumImpl.java,v 2.0 2002/02/15 13:55:29 hnetynka Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.IOException;

public class ExprEnumImpl extends UnicastRemoteObject implements ExprEnum, TIRImplObject {
  EnumDef en;
  CDLType loadEn;
  String val;
  ExprOperKindImpl eok;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;

  public ExprEnumImpl(EnumDef enum, String value) throws RemoteException {
    en = enum;
    val = value;
    eok = new ExprOperKindImpl(ExprOperKind.eok_enum);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Expression);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    loadEn = null;
  }

  public ExprEnumImpl() throws RemoteException {
    en = null;
    loadEn = null;
    val = null;
    eok = new ExprOperKindImpl(ExprOperKind.eok_enum);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Expression);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
  }
 
  public EnumDef enum() throws RemoteException {
    return en;
  }
  
  public String value() throws RemoteException {
    return val;
  }
  
  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public ExprOperKind get_eok_kind() throws RemoteException {
    return (ExprOperKind) eok;
  }
  
  public ExprOperDef calculate() throws RemoteException, TIRExceptCalculate {
    throw new TIRExceptCalculate("Can't calculate enum.");
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      st.curOutFile.writeInt(eok.value());
      Storage.writeCDLType(st,en);
      Storage.writeString(st.curOutFile,val);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in directory "+st.current+".");
    }
  }
  
  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      loadEn = Storage.readCDLType(st);
      val = Storage.readString(st.curInFile);     
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't read in directory "+st.current+".");
    }
  }
  
  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    try {
    
    if (loadEn.get_def_kind().value() == DefinitionKind.dk_none) {   // is NotYetInRep?
      AbsoluteName nm = ((NotYetInRep) loadEn).nm;
      Container con = r;// begin search from repository
      Contained[] rt;
      Contained ret;
      IdentificationImpl id;
      for (int i=0;i<nm.size()-1;i++) {  // last element is name of constant
	id = new IdentificationImpl(nm.elementAt(0),"::"+nm.elementAt(i),"");
        rt = ((SContainer)con).slookup_name(id);
	if (rt.length==0) throw new TIRExceptStorage("Can't find type " + nm.name());
        if (rt.length==1) {
          con = (Container) rt[0];
	} else {  
	  boolean found = false;
          id.version(((NotYetInRep) loadEn).id.version());
          int j = 0;
	  for (j=0;j<rt.length;j++) {
	    if (id.is_short_equal(rt[j].get_identification())) {
	      found = true;
              break;  // -- break for
	    }
	  } // -- end of for
	  if (!found) throw new TIRExceptStorage("Can't find type " + nm.name());
	  con = (Container) rt[j];
	}
      }
      if ((ret = ((SContainer)con).slookup(((NotYetInRep) loadEn).id))==null)
      throw new TIRExceptStorage("Can't find type " + nm.name());
      en = (EnumDef) ret;
    } else {
      en = (EnumDef) loadEn;
    }  
    loadEn = null;

    } catch (TIRExceptLock e) {;}
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_work) {
      stKindImpl.toNormal();
      Identification idl = en.get_identification();
      loadEn = new NotYetInRep(idl.lang_absolute_name().name(),(idl).name(), idl.version());
    }
  }

  public void doAbort(long workId) throws RemoteException {}
}
