/* $Id: FixedDefImpl.java,v 2.0 2002/02/15 13:55:32 hnetynka Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.*;

public class FixedDefImpl extends UnicastRemoteObject implements FixedDef, TIRImplObject {
  ExprOperDef dig;
  ExprOperDef sc;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;

  public FixedDefImpl(ExprOperDef digits, ExprOperDef scale) throws RemoteException {
    dig = digits;
    sc = scale;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Fixed);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
  }

  public FixedDefImpl() throws RemoteException {
    dig = null;
    sc = null;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Fixed);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
  }
  
  public ExprOperDef digits() throws RemoteException {
    return dig;
  }
  
  public ExprOperDef scale() throws RemoteException {
    return sc;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }
  
  /* load & save OK */
  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    dig = Storage.readExprOperDef(st);
    sc = Storage.readExprOperDef(st);
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      Storage.writeExprOperDef(st,dig);
      Storage.writeExprOperDef(st,sc);
    } catch (IOException e) {
      throw new  TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    ((TIRImplObject) dig).postLoad(r);
    ((TIRImplObject) sc).postLoad(r);
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    if (isNew()) {
      stKindImpl.toNormal();
      ((TIRImplObject) dig).doCommit(in,rep);
      ((TIRImplObject) sc).doCommit(in,rep);
    }
  }

  public void doAbort(long workId) throws RemoteException {}
}
